@@ -1,4 +1,4 @@
-__version__ = "0.8.3"
+__version__ = "0.14.7"
 
 from bigtree.binarytree.construct import list_to_binarytree
 from bigtree.dag.construct import dataframe_to_dag, dict_to_dag, list_to_dag
@@ -26,27 +26,34 @@ from bigtree.tree.export import (
     tree_to_dataframe,
     tree_to_dict,
     tree_to_dot,
+    tree_to_mermaid,
     tree_to_nested_dict,
     tree_to_pillow,
     yield_tree,
 )
 from bigtree.tree.helper import clone_tree, get_tree_diff, prune_tree
 from bigtree.tree.modify import (
+    copy_and_replace_nodes_from_tree_to_tree,
     copy_nodes,
     copy_nodes_from_tree_to_tree,
     copy_or_shift_logic,
+    replace_logic,
+    shift_and_replace_nodes,
     shift_nodes,
 )
 from bigtree.tree.search import (
     find,
     find_attr,
     find_attrs,
+    find_child,
+    find_child_by_name,
     find_children,
     find_full_path,
     find_name,
     find_names,
     find_path,
     find_paths,
+    find_relative_path,
     findall,
 )
 from bigtree.utils.exceptions import (
@@ -57,6 +64,7 @@ from bigtree.utils.exceptions import (
     SearchError,
     TreeError,
 )
+from bigtree.utils.groot import speak_like_groot, whoami
 from bigtree.utils.iterators import (
     dag_iterator,
     inorder_iter,
@@ -64,5 +72,9 @@ from bigtree.utils.iterators import (
     levelordergroup_iter,
     postorder_iter,
     preorder_iter,
+    zigzag_iter,
+    zigzaggroup_iter,
 )
+from bigtree.utils.plot import reingold_tilford
+from bigtree.workflows.app_calendar import Calendar
 from bigtree.workflows.app_todo import AppToDo
