@@ -8,12 +8,12 @@ __all__ = ["list_to_binarytree"]
 def list_to_binarytree(
     heapq_list: List[int], node_type: Type[BinaryNode] = BinaryNode
 ) -> BinaryNode:
-    """Construct tree from list of numbers (int or float) in heapq format.
+    """Construct tree from a list of numbers (int or float) in heapq format.
 
-    >>> from bigtree import list_to_binarytree, print_tree, tree_to_dot
+    >>> from bigtree import list_to_binarytree, tree_to_dot
     >>> nums_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
     >>> root = list_to_binarytree(nums_list)
-    >>> print_tree(root)
+    >>> root.show()
     1
     ├── 2
     │   ├── 4
@@ -31,7 +31,7 @@ def list_to_binarytree(
 
     Args:
         heapq_list (List[int]): list containing integer node names, ordered in heapq fashion
-        node_type (Type[BinaryNode]): node type of tree to be created, defaults to BinaryNode
+        node_type (Type[BinaryNode]): node type of tree to be created, defaults to ``BinaryNode``
 
     Returns:
         (BinaryNode)
