@@ -1,18 +1,24 @@
+from __future__ import annotations
+
 from typing import Any, Dict, List, Tuple, Type
 
-import numpy as np
-import pandas as pd
-
 from bigtree.node.dagnode import DAGNode
+from bigtree.utils.exceptions import optional_dependencies_pandas
+
+try:
+    import pandas as pd
+except ImportError:  # pragma: no cover
+    pd = None
 
 __all__ = ["list_to_dag", "dict_to_dag", "dataframe_to_dag"]
 
 
+@optional_dependencies_pandas
 def list_to_dag(
     relations: List[Tuple[str, str]],
     node_type: Type[DAGNode] = DAGNode,
 ) -> DAGNode:
-    """Construct DAG from list of tuple containing parent-child names.
+    """Construct DAG from list of tuples containing parent-child names.
     Note that node names must be unique.
 
     >>> from bigtree import list_to_dag, dag_iterator
@@ -23,7 +29,7 @@ def list_to_dag(
 
     Args:
         relations (List[Tuple[str, str]]): list containing tuple of parent-child names
-        node_type (Type[DAGNode]): node type of DAG to be created, defaults to DAGNode
+        node_type (Type[DAGNode]): node type of DAG to be created, defaults to ``DAGNode``
 
     Returns:
         (DAGNode)
@@ -42,8 +48,8 @@ def dict_to_dag(
     parent_key: str = "parents",
     node_type: Type[DAGNode] = DAGNode,
 ) -> DAGNode:
-    """Construct DAG from nested dictionary, ``key``: child name, ``value``: dict of parent names, attribute name and
-    attribute value.
+    """Construct DAG from nested dictionary, ``key``: child name, ``value``: dictionary of parent names, attribute
+    name, and attribute value.
     Note that node names must be unique.
 
     >>> from bigtree import dict_to_dag, dag_iterator
@@ -60,9 +66,9 @@ def dict_to_dag(
 
     Args:
         relation_attrs (Dict[str, Any]): dictionary containing node, node parents, and node attribute information,
-            key: child name, value: dict of parent names, node attribute and attribute value
+            key: child name, value: dictionary of parent names, node attribute, and attribute value
         parent_key (str): key of dictionary to retrieve list of parents name, defaults to 'parent'
-        node_type (Type[DAGNode]): node type of DAG to be created, defaults to DAGNode
+        node_type (Type[DAGNode]): node type of DAG to be created, defaults to ``DAGNode``
 
     Returns:
         (DAGNode)
@@ -72,9 +78,10 @@ def dict_to_dag(
 
     # Convert dictionary to dataframe
     data = pd.DataFrame(relation_attrs).T.rename_axis("_tmp_child").reset_index()
-    assert (
-        parent_key in data
-    ), f"Parent key {parent_key} not in dictionary, check `relation_attrs` and `parent_key`"
+    if parent_key not in data:
+        raise ValueError(
+            f"Parent key {parent_key} not in dictionary, check `relation_attrs` and `parent_key`"
+        )
 
     data = data.explode(parent_key)
     return dataframe_to_dag(
@@ -85,6 +92,7 @@ def dict_to_dag(
     )
 
 
+@optional_dependencies_pandas
 def dataframe_to_dag(
     data: pd.DataFrame,
     child_col: str = "",
@@ -125,7 +133,7 @@ def dataframe_to_dag(
             if not set, it will take the second column of data
         attribute_cols (List[str]): columns of data containing child node attribute information,
             if not set, it will take all columns of data except `child_col` and `parent_col`
-        node_type (Type[DAGNode]): node type of DAG to be created, defaults to DAGNode
+        node_type (Type[DAGNode]): node type of DAG to be created, defaults to ``DAGNode``
 
     Returns:
         (DAGNode)
@@ -139,14 +147,24 @@ def dataframe_to_dag(
 
     if not child_col:
         child_col = data.columns[0]
+    elif child_col not in data.columns:
+        raise ValueError(f"Child column not in data, check `child_col`: {child_col}")
     if not parent_col:
         parent_col = data.columns[1]
+    elif parent_col not in data.columns:
+        raise ValueError(f"Parent column not in data, check `parent_col`: {parent_col}")
     if not len(attribute_cols):
         attribute_cols = list(data.columns)
         attribute_cols.remove(child_col)
         attribute_cols.remove(parent_col)
+    elif any([col not in data.columns for col in attribute_cols]):
+        raise ValueError(
+            f"One or more attribute column(s) not in data, check `attribute_cols`: {attribute_cols}"
+        )
 
-    data_check = data.copy()[[child_col] + attribute_cols].drop_duplicates()
+    data_check = data.copy()[[child_col, parent_col] + attribute_cols].drop_duplicates(
+        subset=[child_col] + attribute_cols
+    )
     _duplicate_check = (
         data_check[child_col]
         .value_counts()
@@ -157,10 +175,11 @@ def dataframe_to_dag(
     _duplicate_check = _duplicate_check[_duplicate_check["counts"] > 1]
     if len(_duplicate_check):
         raise ValueError(
-            f"There exists duplicate child name with different attributes\nCheck {_duplicate_check}"
+            f"There exists duplicate child name with different attributes\n"
+            f"Check {_duplicate_check}"
         )
-    if np.any(data[child_col].isnull()):
-        raise ValueError(f"Child name cannot be empty, check {child_col}")
+    if sum(data[child_col].isnull()):
+        raise ValueError(f"Child name cannot be empty, check column: {child_col}")
 
     node_dict: Dict[str, DAGNode] = dict()
     parent_node = DAGNode()
