@@ -1,17 +1,34 @@
-from typing import Any, Dict, List, Tuple, Union
+from __future__ import annotations
 
-import pandas as pd
+from typing import Any, Dict, List, Tuple, TypeVar, Union
 
 from bigtree.node.dagnode import DAGNode
+from bigtree.utils.exceptions import (
+    optional_dependencies_image,
+    optional_dependencies_pandas,
+)
 from bigtree.utils.iterators import dag_iterator
 
+try:
+    import pandas as pd
+except ImportError:  # pragma: no cover
+    pd = None
+
+try:
+    import pydot
+except ImportError:  # pragma: no cover
+    pydot = None
+
 __all__ = ["dag_to_list", "dag_to_dict", "dag_to_dataframe", "dag_to_dot"]
 
 
+T = TypeVar("T", bound=DAGNode)
+
+
 def dag_to_list(
-    dag: DAGNode,
+    dag: T,
 ) -> List[Tuple[str, str]]:
-    """Export DAG to list of tuple containing parent-child names
+    """Export DAG to list of tuples containing parent-child names
 
     >>> from bigtree import DAGNode, dag_to_list
     >>> a = DAGNode("a", step=1)
@@ -35,12 +52,12 @@ def dag_to_list(
 
 
 def dag_to_dict(
-    dag: DAGNode,
+    dag: T,
     parent_key: str = "parents",
     attr_dict: Dict[str, str] = {},
     all_attrs: bool = False,
 ) -> Dict[str, Any]:
-    """Export tree to dictionary.
+    """Export DAG to dictionary.
 
     Exported dictionary will have key as child name, and parent names and node attributes as a nested dictionary.
 
@@ -95,8 +112,9 @@ def dag_to_dict(
     return data_dict
 
 
+@optional_dependencies_pandas
 def dag_to_dataframe(
-    dag: DAGNode,
+    dag: T,
     name_col: str = "name",
     parent_col: str = "parent",
     attr_dict: Dict[str, str] = {},
@@ -160,18 +178,20 @@ def dag_to_dataframe(
     return pd.DataFrame(data_list).drop_duplicates().reset_index(drop=True)
 
 
-def dag_to_dot(  # type: ignore[no-untyped-def]
-    dag: Union[DAGNode, List[DAGNode]],
+@optional_dependencies_image("pydot")
+def dag_to_dot(
+    dag: Union[T, List[T]],
     rankdir: str = "TB",
     bg_colour: str = "",
     node_colour: str = "",
+    node_shape: str = "",
     edge_colour: str = "",
     node_attr: str = "",
     edge_attr: str = "",
-):
-    r"""Export DAG tree or list of DAG trees to image.
+) -> pydot.Dot:
+    r"""Export DAG or list of DAGs to image.
     Note that node names must be unique.
-    Posible node attributes include style, fillcolor, shape.
+    Possible node attributes include style, fillcolor, shape.
 
     >>> from bigtree import DAGNode, dag_to_dot
     >>> a = DAGNode("a", step=1)
@@ -181,6 +201,12 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
     >>> e = DAGNode("e", step=3, parents=[d])
     >>> dag_graph = dag_to_dot(a)
 
+    Display image directly without saving (requires IPython)
+
+    >>> from IPython.display import Image, display
+    >>> plt = Image(dag_graph.create_png())
+    >>> display(plt)
+
     Export to image, dot file, etc.
 
     >>> dag_graph.write_png("tree_dag.png")
@@ -196,6 +222,8 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
         rankdir (str): set direction of graph layout, defaults to 'TB', can be 'BT, 'LR', 'RL'
         bg_colour (str): background color of image, defaults to ''
         node_colour (str): fill colour of nodes, defaults to ''
+        node_shape (str): shape of nodes, defaults to None
+            Possible node_shape include "circle", "square", "diamond", "triangle"
         edge_colour (str): colour of edges, defaults to ''
         node_attr (str): node attribute for style, overrides node_colour, defaults to ''
             Possible node attributes include {"style": "filled", "fillcolor": "gold"}
@@ -205,13 +233,6 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
     Returns:
         (pydot.Dot)
     """
-    try:
-        import pydot
-    except ImportError:  # pragma: no cover
-        raise ImportError(
-            "pydot not available. Please perform a\n\npip install 'bigtree[image]'\n\nto install required dependencies"
-        )
-
     # Get style
     if bg_colour:
         graph_style = dict(bgcolor=bg_colour)
@@ -223,6 +244,9 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
     else:
         node_style = dict()
 
+    if node_shape:
+        node_style["shape"] = node_shape
+
     if edge_colour:
         edge_style = dict(color=edge_colour)
     else:
@@ -237,21 +261,21 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
 
     for _dag in dag:
         if not isinstance(_dag, DAGNode):
-            raise ValueError(
+            raise TypeError(
                 "Tree should be of type `DAGNode`, or inherit from `DAGNode`"
             )
         _dag = _dag.copy()
 
         for parent_node, child_node in dag_iterator(_dag):
+            _node_style = node_style.copy()
+            _edge_style = edge_style.copy()
+
             child_name = child_node.name
-            child_node_style = node_style.copy()
             if node_attr and child_node.get_attr(node_attr):
-                child_node_style.update(child_node.get_attr(node_attr))
-            if edge_attr:
-                edge_style.update(child_node.get_attr(edge_attr))
-            pydot_child = pydot.Node(
-                name=child_name, label=child_name, **child_node_style
-            )
+                _node_style.update(child_node.get_attr(node_attr))
+            if edge_attr and child_node.get_attr(edge_attr):
+                _edge_style.update(child_node.get_attr(edge_attr))
+            pydot_child = pydot.Node(name=child_name, label=child_name, **_node_style)
             _graph.add_node(pydot_child)
 
             parent_name = parent_node.name
@@ -263,7 +287,7 @@ def dag_to_dot(  # type: ignore[no-untyped-def]
             )
             _graph.add_node(pydot_parent)
 
-            edge = pydot.Edge(parent_name, child_name, **edge_style)
+            edge = pydot.Edge(parent_name, child_name, **_edge_style)
             _graph.add_edge(edge)
 
     return _graph
