@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import copy
-from typing import Any, Dict, Iterable, List, Optional, Tuple, TypeVar
+from typing import Any, Dict, Generator, Iterable, List, Optional, Set, Tuple, TypeVar
 
 from bigtree.utils.exceptions import CorruptedTreeError, LoopError, TreeError
 from bigtree.utils.iterators import preorder_iter
@@ -103,12 +103,14 @@ class BaseNode:
 
     `BaseNode` methods
 
-    1. ``describe()``: Get node information sorted by attributes, returns list of tuples
+    1. ``describe()``: Get node information sorted by attributes, return list of tuples
     2. ``get_attr(attr_name: str)``: Get value of node attribute
     3. ``set_attrs(attrs: dict)``: Set node attribute name(s) and value(s)
     4. ``go_to(node: Self)``: Get a path from own node to another node from same tree
-    5. ``copy()``: Deep copy self
-    6. ``sort()``: Sort child nodes
+    5. ``append(node: Self)``: Add child to node
+    6. ``extend(nodes: List[Self])``: Add multiple children to node
+    7. ``copy()``: Deep copy self
+    8. ``sort()``: Sort child nodes
 
     ----
 
@@ -127,7 +129,7 @@ class BaseNode:
         self.parent = parent
         self.children = children  # type: ignore
         if "parents" in kwargs:
-            raise ValueError(
+            raise AttributeError(
                 "Attempting to set `parents` attribute, do you mean `parent`?"
             )
         self.__dict__.update(**kwargs)
@@ -141,7 +143,7 @@ class BaseNode:
         """
         if not (isinstance(new_parent, BaseNode) or new_parent is None):
             raise TypeError(
-                f"Expect input to be BaseNode type or NoneType, received input type {type(new_parent)}"
+                f"Expect parent to be BaseNode type or NoneType, received input type {type(new_parent)}"
             )
 
     def __check_parent_loop(self, new_parent: T) -> None:
@@ -218,7 +220,7 @@ class BaseNode:
 
     def __pre_assign_parent(self, new_parent: T) -> None:
         """Custom method to check before attaching parent
-        Can be overriden with `_BaseNode__pre_assign_parent()`
+        Can be overridden with `_BaseNode__pre_assign_parent()`
 
         Args:
             new_parent (Self): new parent to be added
@@ -227,7 +229,7 @@ class BaseNode:
 
     def __post_assign_parent(self, new_parent: T) -> None:
         """Custom method to check after attaching parent
-        Can be overriden with `_BaseNode__post_assign_parent()`
+        Can be overridden with `_BaseNode__post_assign_parent()`
 
         Args:
             new_parent (Self): new parent to be added
@@ -237,7 +239,7 @@ class BaseNode:
     @property
     def parents(self) -> None:
         """Do not allow `parents` attribute to be accessed"""
-        raise ValueError(
+        raise AttributeError(
             "Attempting to access `parents` attribute, do you mean `parent`?"
         )
 
@@ -248,31 +250,39 @@ class BaseNode:
         Args:
             new_parent (Self): parent node
         """
-        raise ValueError("Attempting to set `parents` attribute, do you mean `parent`?")
+        raise AttributeError(
+            "Attempting to set `parents` attribute, do you mean `parent`?"
+        )
 
-    def __check_children_type(self: T, new_children: List[T]) -> None:
+    def __check_children_type(
+        self: T, new_children: List[T] | Tuple[T] | Set[T]
+    ) -> None:
         """Check child type
 
         Args:
-            new_children (List[Self]): child node
+            new_children (Iterable[Self]): child node
         """
-        if not isinstance(new_children, list):
+        if (
+            not isinstance(new_children, list)
+            and not isinstance(new_children, tuple)
+            and not isinstance(new_children, set)
+        ):
             raise TypeError(
-                f"Children input should be list type, received input type {type(new_children)}"
+                f"Expect children to be List or Tuple or Set type, received input type {type(new_children)}"
             )
 
-    def __check_children_loop(self: T, new_children: List[T]) -> None:
+    def __check_children_loop(self: T, new_children: Iterable[T]) -> None:
         """Check child loop
 
         Args:
-            new_children (List[Self]): child node
+            new_children (Iterable[Self]): child node
         """
         seen_children = []
         for new_child in new_children:
             # Check type
             if not isinstance(new_child, BaseNode):
                 raise TypeError(
-                    f"Expect input to be BaseNode type, received input type {type(new_child)}"
+                    f"Expect children to be BaseNode type, received input type {type(new_child)}"
                 )
 
             # Check for loop and tree structure
@@ -280,7 +290,7 @@ class BaseNode:
                 raise LoopError("Error setting child: Node cannot be child of itself")
             if any(child is new_child for child in self.ancestors):
                 raise LoopError(
-                    "Error setting child: Node cannot be ancestors of itself"
+                    "Error setting child: Node cannot be ancestor of itself"
                 )
 
             # Check for duplicate children
@@ -301,7 +311,7 @@ class BaseNode:
         return tuple(self.__children)
 
     @children.setter
-    def children(self: T, new_children: List[T]) -> None:
+    def children(self: T, new_children: List[T] | Tuple[T] | Set[T]) -> None:
         """Set child nodes
 
         Args:
@@ -309,6 +319,7 @@ class BaseNode:
         """
         self.__check_children_type(new_children)
         self.__check_children_loop(new_children)
+        new_children = list(new_children)
 
         current_new_children = {
             new_child: (new_child.parent.__children.index(new_child), new_child.parent)
@@ -355,21 +366,21 @@ class BaseNode:
             child.parent.__children.remove(child)  # type: ignore
             child.__parent = None
 
-    def __pre_assign_children(self: T, new_children: List[T]) -> None:
+    def __pre_assign_children(self: T, new_children: Iterable[T]) -> None:
         """Custom method to check before attaching children
-        Can be overriden with `_BaseNode__pre_assign_children()`
+        Can be overridden with `_BaseNode__pre_assign_children()`
 
         Args:
-            new_children (List[Self]): new children to be added
+            new_children (Iterable[Self]): new children to be added
         """
         pass
 
-    def __post_assign_children(self: T, new_children: List[T]) -> None:
+    def __post_assign_children(self: T, new_children: Iterable[T]) -> None:
         """Custom method to check after attaching children
-        Can be overriden with `_BaseNode__post_assign_children()`
+        Can be overridden with `_BaseNode__post_assign_children()`
 
         Args:
-            new_children (List[Self]): new children to be added
+            new_children (Iterable[Self]): new children to be added
         """
         pass
 
@@ -546,9 +557,9 @@ class BaseNode:
             and (not len(exclude_prefix) or not item[0].startswith(exclude_prefix))
         ]
 
-    def get_attr(self, attr_name: str) -> Any:
+    def get_attr(self, attr_name: str, default_value: Any = None) -> Any:
         """Get value of node attribute
-        Returns None if attribute name does not exist
+        Returns default value if attribute name does not exist
 
         >>> from bigtree.node.node import Node
         >>> a = Node('a', age=90)
@@ -557,14 +568,15 @@ class BaseNode:
 
         Args:
             attr_name (str): attribute name
+            default_value (Any): default value if attribute does not exist, defaults to None
 
         Returns:
             (Any)
         """
         try:
-            return self.__getattribute__(attr_name)
+            return getattr(self, attr_name)
         except AttributeError:
-            return None
+            return default_value
 
     def set_attrs(self, attrs: Dict[str, Any]) -> None:
         """Set node attributes
@@ -634,6 +646,23 @@ class BaseNode:
         node_min_index = node_path.index(min_common_node)
         return self_path[:self_min_index] + node_path[node_min_index:]
 
+    def append(self: T, other: T) -> None:
+        """Add other as child of self
+
+        Args:
+            other (Self): other node, child to be added
+        """
+        other.parent = self
+
+    def extend(self: T, others: List[T]) -> None:
+        """Add others as children of self
+
+        Args:
+            others (Self): other nodes, children to be added
+        """
+        for child in others:
+            child.parent = self
+
     def copy(self: T) -> T:
         """Deep copy self; clone self
 
@@ -694,7 +723,7 @@ class BaseNode:
         return f"{class_name}({node_description})"
 
     def __rshift__(self: T, other: T) -> None:
-        """Set children using >> bitshift operator for self >> other
+        """Set children using >> bitshift operator for self >> children (other)
 
         Args:
             other (Self): other node, children
@@ -702,12 +731,31 @@ class BaseNode:
         other.parent = self
 
     def __lshift__(self: T, other: T) -> None:
-        """Set parent using << bitshift operator for self << other
+        """Set parent using << bitshift operator for self << parent (other)
 
         Args:
             other (Self): other node, parent
         """
         self.parent = other
 
+    def __iter__(self) -> Generator[T, None, None]:
+        """Iterate through child nodes
+
+        Returns:
+            (Self): child node
+        """
+        yield from self.children  # type: ignore
+
+    def __contains__(self, other_node: T) -> bool:
+        """Check if child node exists
+
+        Args:
+            other_node (T): child node
+
+        Returns:
+            (bool)
+        """
+        return other_node in self.children
+
 
 T = TypeVar("T", bound=BaseNode)
