@@ -1,14 +1,22 @@
+from __future__ import annotations
+
 import re
 from collections import OrderedDict
-from typing import Any, Dict, List, Optional, Tuple, Type
-
-import numpy as np
-import pandas as pd
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Type
 
 from bigtree.node.node import Node
 from bigtree.tree.export import tree_to_dataframe
-from bigtree.tree.search import find_children, find_name
-from bigtree.utils.exceptions import DuplicatedNodeError, TreeError
+from bigtree.tree.search import find_child_by_name, find_name
+from bigtree.utils.exceptions import (
+    DuplicatedNodeError,
+    TreeError,
+    optional_dependencies_pandas,
+)
+
+try:
+    import pandas as pd
+except ImportError:  # pragma: no cover
+    pd = None
 
 __all__ = [
     "add_path_to_tree",
@@ -36,21 +44,21 @@ def add_path_to_tree(
     """Add nodes and attributes to existing tree *in-place*, return node of added path.
     Adds to existing tree from list of path strings.
 
-    Path should contain `Node` name, separated by `sep`.
+    Path should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
-      - Path separator `sep` is for the input `path` and can be different from that of existing tree.
+      - Path separator `sep` is for the input `path` and can differ from existing tree.
 
     Path can start from root node `name`, or start with `sep`.
       - For example: Path string can be "/a/b" or "a/b", if sep is "/".
 
     All paths should start from the same root node.
-      - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
+      - For example: Path strings should be "a/b", "a/c", "a/b/d" etc., and should not start with another root node.
 
-    >>> from bigtree import add_path_to_tree, print_tree
+    >>> from bigtree import add_path_to_tree
     >>> root = Node("a")
     >>> add_path_to_tree(root, "a/b/c")
     Node(/a/b/c, )
-    >>> print_tree(root)
+    >>> root.show()
     a
     └── b
         └── c
@@ -59,7 +67,7 @@ def add_path_to_tree(
         tree (Node): existing tree
         path (str): path to be added to tree
         sep (str): path separator for input `path`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
         node_attrs (Dict[str, Any]): attributes to add to node, key: attribute name, value: attribute value, optional
 
     Returns:
@@ -74,7 +82,7 @@ def add_path_to_tree(
     branch = path.lstrip(sep).rstrip(sep).split(sep)
     if branch[0] != tree_root.node_name:
         raise TreeError(
-            f"Error: Path does not have same root node, expected {tree_root.node_name}, received {branch[0]}\n"
+            f"Path does not have same root node, expected {tree_root.node_name}, received {branch[0]}\n"
             f"Check your input paths or verify that path separator `sep` is set correctly"
         )
 
@@ -92,9 +100,13 @@ def add_path_to_tree(
                     f"to allow `Node` with same node name"
                 )
         else:
-            node = find_children(parent_node, node_name)
+            node = find_child_by_name(parent_node, node_name)
         if not node:
-            node = node_type(branch[idx])
+            if idx == len(branch) - 1:
+                node_name = node_attrs.pop("name", branch[idx])
+                node = node_type(node_name, **node_attrs)
+            else:
+                node = node_type(branch[idx])
             node.parent = parent_node
         parent_node = node
     node.set_attrs(node_attrs)
@@ -110,9 +122,9 @@ def add_dict_to_tree_by_path(
     """Add nodes and attributes to tree *in-place*, return root of tree.
     Adds to existing tree from nested dictionary, ``key``: path, ``value``: dict of attribute name and attribute value.
 
-    Path should contain `Node` name, separated by `sep`.
+    Path should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
-      - Path separator `sep` is for the input `path` and can be different from that of existing tree.
+      - Path separator `sep` is for the input `path` and can differ from existing tree.
 
     Path can start from root node `name`, or start with `sep`.
       - For example: Path string can be "/a/b" or "a/b", if sep is "/".
@@ -120,7 +132,7 @@ def add_dict_to_tree_by_path(
     All paths should start from the same root node.
       - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
 
-    >>> from bigtree import Node, add_dict_to_tree_by_path, print_tree
+    >>> from bigtree import Node, add_dict_to_tree_by_path
     >>> root = Node("a")
     >>> path_dict = {
     ...     "a": {"age": 90},
@@ -133,7 +145,7 @@ def add_dict_to_tree_by_path(
     ...     "a/b/e/h": {"age": 6},
     ... }
     >>> root = add_dict_to_tree_by_path(root, path_dict)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -148,7 +160,7 @@ def add_dict_to_tree_by_path(
         path_attrs (Dict[str, Dict[str, Any]]): dictionary containing node path and attribute information,
             key: node path, value: dict of node attribute name and attribute value
         sep (str): path separator for input `path_attrs`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
 
     Returns:
         (Node)
@@ -169,17 +181,18 @@ def add_dict_to_tree_by_path(
     return tree_root
 
 
+@optional_dependencies_pandas
 def add_dict_to_tree_by_name(
-    tree: Node, path_attrs: Dict[str, Dict[str, Any]], join_type: str = "left"
+    tree: Node, name_attrs: Dict[str, Dict[str, Any]], join_type: str = "left"
 ) -> Node:
     """Add attributes to tree, return *new* root of tree.
     Adds to existing tree from nested dictionary, ``key``: name, ``value``: dict of attribute name and attribute value.
 
-    Function can return all existing tree nodes or only tree nodes that are in the input dictionary keys.
+    Function can return all existing tree nodes or only tree nodes that are in the input dictionary keys depending on join type.
     Input dictionary keys that are not existing node names will be ignored.
-    Note that if multiple nodes have the same name, attributes will be added to all nodes sharing same name.
+    Note that if multiple nodes have the same name, attributes will be added to all nodes sharing the same name.
 
-    >>> from bigtree import Node, add_dict_to_tree_by_name, print_tree
+    >>> from bigtree import Node, add_dict_to_tree_by_name
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> name_dict = {
@@ -187,16 +200,16 @@ def add_dict_to_tree_by_name(
     ...     "b": {"age": 65},
     ... }
     >>> root = add_dict_to_tree_by_name(root, name_dict)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     └── b [age=65]
 
     Args:
         tree (Node): existing tree
-        path_attrs (Dict[str, Dict[str, Any]]): dictionary containing node name and attribute information,
+        name_attrs (Dict[str, Dict[str, Any]]): dictionary containing node name and attribute information,
             key: node name, value: dict of node attribute name and attribute value
         join_type (str): join type with attribute, default of 'left' takes existing tree nodes,
-            if join_type is set to 'inner' it will only take tree nodes that are in `path_attrs` key and drop others
+            if join_type is set to 'inner' it will only take tree nodes that are in `name_attrs` key and drop others
 
     Returns:
         (Node)
@@ -204,11 +217,11 @@ def add_dict_to_tree_by_name(
     if join_type not in ["inner", "left"]:
         raise ValueError("`join_type` must be one of 'inner' or 'left'")
 
-    if not len(path_attrs):
-        raise ValueError("Dictionary does not contain any data, check `path_attrs`")
+    if not len(name_attrs):
+        raise ValueError("Dictionary does not contain any data, check `name_attrs`")
 
     # Convert dictionary to dataframe
-    data = pd.DataFrame(path_attrs).T.rename_axis("NAME").reset_index()
+    data = pd.DataFrame(name_attrs).T.rename_axis("NAME").reset_index()
     return add_dataframe_to_tree_by_name(tree, data=data, join_type=join_type)
 
 
@@ -225,9 +238,9 @@ def add_dataframe_to_tree_by_path(
     `path_col` and `attribute_cols` specify columns for node path and attributes to add to existing tree.
     If columns are not specified, `path_col` takes first column and all other columns are `attribute_cols`
 
-    Path in path column should contain `Node` name, separated by `sep`.
+    Path in path column should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
-      - Path separator `sep` is for the input `path_col` and can be different from that of existing tree.
+      - Path separator `sep` is for the input `path` and can differ from existing tree.
 
     Path in path column can start from root node `name`, or start with `sep`.
       - For example: Path string can be "/a/b" or "a/b", if sep is "/".
@@ -236,7 +249,7 @@ def add_dataframe_to_tree_by_path(
       - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
 
     >>> import pandas as pd
-    >>> from bigtree import add_dataframe_to_tree_by_path, print_tree
+    >>> from bigtree import add_dataframe_to_tree_by_path
     >>> root = Node("a")
     >>> path_data = pd.DataFrame([
     ...     ["a", 90],
@@ -251,7 +264,7 @@ def add_dataframe_to_tree_by_path(
     ...     columns=["PATH", "age"]
     ... )
     >>> root = add_dataframe_to_tree_by_path(root, path_data)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     ├── b [age=65]
     │   ├── d [age=40]
@@ -269,7 +282,7 @@ def add_dataframe_to_tree_by_path(
         attribute_cols (List[str]): columns of data containing node attribute information,
             if not set, it will take all columns of data except `path_col`
         sep (str): path separator for input `path_col`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
 
     Returns:
         (Node)
@@ -306,7 +319,7 @@ def add_dataframe_to_tree_by_path(
     for row in data.to_dict(orient="index").values():
         node_attrs = row.copy()
         del node_attrs[path_col]
-        node_attrs = {k: v for k, v in node_attrs.items() if not np.all(pd.isnull(v))}
+        node_attrs = {k: v for k, v in node_attrs.items() if v is not None}
         add_path_to_tree(
             tree_root,
             row[path_col],
@@ -317,6 +330,7 @@ def add_dataframe_to_tree_by_path(
     return tree_root
 
 
+@optional_dependencies_pandas
 def add_dataframe_to_tree_by_name(
     tree: Node,
     data: pd.DataFrame,
@@ -334,7 +348,7 @@ def add_dataframe_to_tree_by_name(
     Note that if multiple nodes have the same name, attributes will be added to all nodes sharing same name.
 
     >>> import pandas as pd
-    >>> from bigtree import add_dataframe_to_tree_by_name, print_tree
+    >>> from bigtree import add_dataframe_to_tree_by_name
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> name_data = pd.DataFrame([
@@ -344,7 +358,7 @@ def add_dataframe_to_tree_by_name(
     ...     columns=["NAME", "age"]
     ... )
     >>> root = add_dataframe_to_tree_by_name(root, name_data)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     └── b [age=65]
 
@@ -354,7 +368,7 @@ def add_dataframe_to_tree_by_name(
         name_col (str): column of data containing `name` information,
             if not set, it will take the first column of data
         attribute_cols (List[str]): column(s) of data containing node attribute information,
-            if not set, it will take all columns of data except path_col
+            if not set, it will take all columns of data except `path_col`
         join_type (str): join type with attribute, default of 'left' takes existing tree nodes,
             if join_type is set to 'inner' it will only take tree nodes with attributes and drop the other nodes
 
@@ -419,10 +433,10 @@ def str_to_tree(
 ) -> Node:
     r"""Construct tree from tree string
 
-    >>> from bigtree import str_to_tree, print_tree
+    >>> from bigtree import str_to_tree
     >>> tree_str = 'a\n├── b\n│   ├── d\n│   └── e\n│       ├── g\n│       └── h\n└── c\n    └── f'
     >>> root = str_to_tree(tree_str, tree_prefix_list=["├──", "└──"])
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -436,7 +450,7 @@ def str_to_tree(
         tree_string (str): String to construct tree
         tree_prefix_list (List[str]): List of prefix to mark the end of tree branch/stem and start of node name, optional.
             If not specified, it will infer unicode characters and whitespace as prefix.
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
@@ -481,14 +495,14 @@ def str_to_tree(
 
 
 def list_to_tree(
-    paths: List[str],
+    paths: Iterable[str],
     sep: str = "/",
     duplicate_name_allowed: bool = True,
     node_type: Type[Node] = Node,
 ) -> Node:
     """Construct tree from list of path strings.
 
-    Path should contain `Node` name, separated by `sep`.
+    Path should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
 
     Path can start from root node `name`, or start with `sep`.
@@ -497,10 +511,10 @@ def list_to_tree(
     All paths should start from the same root node.
       - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
 
-    >>> from bigtree import list_to_tree, print_tree
+    >>> from bigtree import list_to_tree
     >>> path_list = ["a/b", "a/c", "a/b/d", "a/b/e", "a/c/f", "a/b/e/g", "a/b/e/h"]
     >>> root = list_to_tree(path_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -511,15 +525,15 @@ def list_to_tree(
         └── f
 
     Args:
-        paths (List[str]): list containing path strings
+        paths (Iterable[str]): list containing path strings
         sep (str): path separator for input `paths` and created tree, defaults to `/`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
     """
-    if not len(paths):
+    if not paths:
         raise ValueError("Path list does not contain any data, check `paths`")
 
     # Remove duplicates
@@ -538,19 +552,22 @@ def list_to_tree(
     return root_node
 
 
+@optional_dependencies_pandas
 def list_to_tree_by_relation(
-    relations: List[Tuple[str, str]],
+    relations: Iterable[Tuple[str, str]],
+    allow_duplicates: bool = False,
     node_type: Type[Node] = Node,
 ) -> Node:
     """Construct tree from list of tuple containing parent-child names.
 
-    Note that node names must be unique since tree is created from parent-child names,
-    except for leaf nodes - names of leaf nodes may be repeated as there is no confusion.
+    Since tree is created from parent-child names, only names of leaf nodes may be repeated.
+    Error will be thrown if names of intermediate nodes are repeated as there will be confusion.
+    This error can be ignored by setting `allow_duplicates` to be True.
 
-    >>> from bigtree import list_to_tree_by_relation, print_tree
+    >>> from bigtree import list_to_tree_by_relation
     >>> relations_list = [("a", "b"), ("a", "c"), ("b", "d"), ("b", "e"), ("c", "f"), ("e", "g"), ("e", "h")]
     >>> root = list_to_tree_by_relation(relations_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -561,21 +578,28 @@ def list_to_tree_by_relation(
         └── f
 
     Args:
-        relations (List[Tuple[str, str]]): list containing tuple containing parent-child names
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        relations (Iterable[Tuple[str, str]]): list containing tuple containing parent-child names
+        allow_duplicates (bool): allow duplicate intermediate nodes such that child node will
+            be tagged to multiple parent nodes, defaults to False
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
     """
-    if not len(relations):
+    if not relations:
         raise ValueError("Path list does not contain any data, check `relations`")
 
     relation_data = pd.DataFrame(relations, columns=["parent", "child"])
     return dataframe_to_tree_by_relation(
-        relation_data, child_col="child", parent_col="parent", node_type=node_type
+        relation_data,
+        child_col="child",
+        parent_col="parent",
+        allow_duplicates=allow_duplicates,
+        node_type=node_type,
     )
 
 
+@optional_dependencies_pandas
 def dict_to_tree(
     path_attrs: Dict[str, Any],
     sep: str = "/",
@@ -585,7 +609,7 @@ def dict_to_tree(
     """Construct tree from nested dictionary using path,
     ``key``: path, ``value``: dict of attribute name and attribute value.
 
-    Path should contain `Node` name, separated by `sep`.
+    Path should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
 
     Path can start from root node `name`, or start with `sep`.
@@ -594,7 +618,7 @@ def dict_to_tree(
     All paths should start from the same root node.
       - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
 
-    >>> from bigtree import dict_to_tree, print_tree
+    >>> from bigtree import dict_to_tree
     >>> path_dict = {
     ...     "a": {"age": 90},
     ...     "a/b": {"age": 65},
@@ -606,7 +630,7 @@ def dict_to_tree(
     ...     "a/b/e/h": {"age": 6},
     ... }
     >>> root = dict_to_tree(path_dict)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     ├── b [age=65]
     │   ├── d [age=40]
@@ -620,8 +644,8 @@ def dict_to_tree(
         path_attrs (Dict[str, Any]): dictionary containing path and node attribute information,
             key: path, value: dict of tree attribute and attribute value
         sep (str): path separator of input `path_attrs` and created tree, defaults to `/`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
@@ -650,7 +674,7 @@ def nested_dict_to_tree(
       - ``value`` of `name_key` (str): node name.
       - ``value`` of `child_key` (List[Dict[str, Any]]): list of dict containing `name_key` and `child_key` (recursive).
 
-    >>> from bigtree import nested_dict_to_tree, print_tree
+    >>> from bigtree import nested_dict_to_tree
     >>> path_dict = {
     ...     "name": "a",
     ...     "age": 90,
@@ -666,7 +690,7 @@ def nested_dict_to_tree(
     ...     ],
     ... }
     >>> root = nested_dict_to_tree(path_dict)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     └── b [age=65]
         ├── d [age=40]
@@ -680,11 +704,13 @@ def nested_dict_to_tree(
             value of `child_key` (List[Dict[str, Any]]): list of dict containing `name_key` and `child_key` (recursive)
         name_key (str): key of node name, value is type str
         child_key (str): key of child list, value is type list
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
     """
+    if not node_attrs:
+        raise ValueError("Dictionary does not contain any data, check `node_attrs`")
 
     def recursive_add_child(
         child_dict: Dict[str, Any], parent_node: Optional[Node] = None
@@ -692,6 +718,10 @@ def nested_dict_to_tree(
         child_dict = child_dict.copy()
         node_name = child_dict.pop(name_key)
         node_children = child_dict.pop(child_key, [])
+        if not isinstance(node_children, List):
+            raise TypeError(
+                f"child_key {child_key} should be List type, received {node_children}"
+            )
         node = node_type(node_name, parent=parent_node, **child_dict)
         for _child in node_children:
             recursive_add_child(_child, parent_node=node)
@@ -717,14 +747,14 @@ def dataframe_to_tree(
     Path in path column can start from root node `name`, or start with `sep`.
       - For example: Path string can be "/a/b" or "a/b", if sep is "/".
 
-    Path in path column should contain `Node` name, separated by `sep`.
+    Path in path column should contain ``Node`` name, separated by `sep`.
       - For example: Path string "a/b" refers to Node("b") with parent Node("a").
 
     All paths should start from the same root node.
       - For example: Path strings should be "a/b", "a/c", "a/b/d" etc. and should not start with another root node.
 
     >>> import pandas as pd
-    >>> from bigtree import dataframe_to_tree, print_tree
+    >>> from bigtree import dataframe_to_tree
     >>> path_data = pd.DataFrame([
     ...     ["a", 90],
     ...     ["a/b", 65],
@@ -738,7 +768,7 @@ def dataframe_to_tree(
     ...     columns=["PATH", "age"]
     ... )
     >>> root = dataframe_to_tree(path_data)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     ├── b [age=65]
     │   ├── d [age=40]
@@ -755,8 +785,8 @@ def dataframe_to_tree(
         attribute_cols (List[str]): columns of data containing node attribute information,
             if not set, it will take all columns of data except `path_col`
         sep (str): path separator of input `path_col` and created tree, defaults to `/`
-        duplicate_name_allowed (bool): indicator if nodes with duplicated `Node` name is allowed, defaults to True
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        duplicate_name_allowed (bool): indicator if nodes with duplicate ``Node`` name is allowed, defaults to True
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
@@ -790,10 +820,20 @@ def dataframe_to_tree(
         )
 
     root_name = data[path_col].values[0].split(sep)[0]
-    root_node = node_type(root_name)
+    root_node_data = data[data[path_col] == root_name]
+    if len(root_node_data):
+        root_node_kwargs = list(
+            root_node_data[attribute_cols].to_dict(orient="index").values()
+        )[0]
+        root_name = root_node_kwargs.pop("name", root_name)
+        root_node = node_type(root_name, **root_node_kwargs)
+    else:
+        root_node = node_type(root_name)
     add_dataframe_to_tree_by_path(
         root_node,
         data,
+        path_col=path_col,
+        attribute_cols=attribute_cols,
         sep=sep,
         duplicate_name_allowed=duplicate_name_allowed,
     )
@@ -806,20 +846,22 @@ def dataframe_to_tree_by_relation(
     child_col: str = "",
     parent_col: str = "",
     attribute_cols: List[str] = [],
+    allow_duplicates: bool = False,
     node_type: Type[Node] = Node,
 ) -> Node:
     """Construct tree from pandas DataFrame using parent and child names, return root of tree.
 
-    Note that node names must be unique since tree is created from parent-child names,
-    except for leaf nodes - names of leaf nodes may be repeated as there is no confusion.
+    Since tree is created from parent-child names, only names of leaf nodes may be repeated.
+    Error will be thrown if names of intermediate nodes are repeated as there will be confusion.
+    This error can be ignored by setting `allow_duplicates` to be True.
 
     `child_col` and `parent_col` specify columns for child name and parent name to construct tree.
-    `attribute_cols` specify columns for node attribute for child name
+    `attribute_cols` specify columns for node attribute for child name.
     If columns are not specified, `child_col` takes first column, `parent_col` takes second column, and all other
     columns are `attribute_cols`.
 
     >>> import pandas as pd
-    >>> from bigtree import dataframe_to_tree_by_relation, print_tree
+    >>> from bigtree import dataframe_to_tree_by_relation
     >>> relation_data = pd.DataFrame([
     ...     ["a", None, 90],
     ...     ["b", "a", 65],
@@ -833,7 +875,7 @@ def dataframe_to_tree_by_relation(
     ...     columns=["child", "parent", "age"]
     ... )
     >>> root = dataframe_to_tree_by_relation(relation_data)
-    >>> print_tree(root, attr_list=["age"])
+    >>> root.show(attr_list=["age"])
     a [age=90]
     ├── b [age=65]
     │   ├── d [age=40]
@@ -851,7 +893,9 @@ def dataframe_to_tree_by_relation(
             if not set, it will take the second column of data
         attribute_cols (List[str]): columns of data containing node attribute information,
             if not set, it will take all columns of data except `child_col` and `parent_col`
-        node_type (Type[Node]): node type of tree to be created, defaults to Node
+        allow_duplicates (bool): allow duplicate intermediate nodes such that child node will
+            be tagged to multiple parent nodes, defaults to False
+        node_type (Type[Node]): node type of tree to be created, defaults to ``Node``
 
     Returns:
         (Node)
@@ -874,21 +918,22 @@ def dataframe_to_tree_by_relation(
 
     data_check = data.copy()[[child_col, parent_col]].drop_duplicates()
     # Filter for child nodes that are parent of other nodes
-    data_check = data_check[data_check[child_col].isin(data_check[parent_col])]
-    _duplicate_check = (
-        data_check[child_col]
-        .value_counts()
-        .to_frame("counts")
-        .rename_axis(child_col)
-        .reset_index()
-    )
-    _duplicate_check = _duplicate_check[_duplicate_check["counts"] > 1]
-    if len(_duplicate_check):
-        raise ValueError(
-            f"There exists duplicate child with different parent where the child is also a parent node.\n"
-            f"Duplicated node names should not happen, but can only exist in leaf nodes to avoid confusion.\n"
-            f"Check {_duplicate_check}"
+    if not allow_duplicates:
+        data_check = data_check[data_check[child_col].isin(data_check[parent_col])]
+        _duplicate_check = (
+            data_check[child_col]
+            .value_counts()
+            .to_frame("counts")
+            .rename_axis(child_col)
+            .reset_index()
         )
+        _duplicate_check = _duplicate_check[_duplicate_check["counts"] > 1]
+        if len(_duplicate_check):
+            raise ValueError(
+                f"There exists duplicate child with different parent where the child is also a parent node.\n"
+                f"Duplicated node names should not happen, but can only exist in leaf nodes to avoid confusion.\n"
+                f"Check {_duplicate_check}"
+            )
 
     # If parent-child contains None -> root
     root_row = data[data[parent_col].isnull()]
@@ -896,16 +941,26 @@ def dataframe_to_tree_by_relation(
     if not len(root_names):
         root_names = list(set(data[parent_col]) - set(data[child_col]))
     if len(root_names) != 1:
-        raise ValueError(f"Unable to determine root node\nCheck {root_names}")
+        raise ValueError(
+            f"Unable to determine root node\nPossible root nodes: {root_names}"
+        )
     root_name = root_names[0]
-    root_node = node_type(root_name)
+    root_node_data = data[data[child_col] == root_name]
+    if len(root_node_data):
+        root_node_kwargs = list(
+            root_node_data[attribute_cols].to_dict(orient="index").values()
+        )[0]
+        root_name = root_node_kwargs.pop("name", root_name)
+        root_node = node_type(root_name, **root_node_kwargs)
+    else:
+        root_node = node_type(root_name)
 
     def retrieve_attr(_row: Dict[str, Any]) -> Dict[str, Any]:
         node_attrs = _row.copy()
         node_attrs["name"] = node_attrs[child_col]
         del node_attrs[child_col]
         del node_attrs[parent_col]
-        _node_attrs = {k: v for k, v in node_attrs.items() if not np.all(pd.isnull(v))}
+        _node_attrs = {k: v for k, v in node_attrs.items() if v is not None}
         return _node_attrs
 
     def recursive_create_child(parent_node: Node) -> None:
