@@ -1,12 +1,35 @@
-import collections
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
+from __future__ import annotations
 
-import pandas as pd
+import collections
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
+from urllib.request import urlopen
 
 from bigtree.node.node import Node
 from bigtree.tree.search import find_path
+from bigtree.utils.assertions import assert_key_in_dict, assert_str_in_list
+from bigtree.utils.constants import ExportConstants, MermaidConstants
+from bigtree.utils.exceptions import (
+    optional_dependencies_image,
+    optional_dependencies_pandas,
+)
 from bigtree.utils.iterators import preorder_iter
 
+try:
+    import pandas as pd
+except ImportError:  # pragma: no cover
+    pd = None
+
+try:
+    import pydot
+except ImportError:  # pragma: no cover
+    pydot = None
+
+try:
+    from PIL import Image, ImageDraw, ImageFont
+except ImportError:  # pragma: no cover
+    Image = ImageDraw = ImageFont = None
+
+
 __all__ = [
     "print_tree",
     "yield_tree",
@@ -15,29 +38,22 @@ __all__ = [
     "tree_to_dataframe",
     "tree_to_dot",
     "tree_to_pillow",
+    "tree_to_mermaid",
 ]
 
-available_styles = {
-    "ansi": ("|   ", "|-- ", "`-- "),
-    "ascii": ("|   ", "|-- ", "+-- "),
-    "const": ("\u2502   ", "\u251c\u2500\u2500 ", "\u2514\u2500\u2500 "),
-    "const_bold": ("\u2503   ", "\u2523\u2501\u2501 ", "\u2517\u2501\u2501 "),
-    "rounded": ("\u2502   ", "\u251c\u2500\u2500 ", "\u2570\u2500\u2500 "),
-    "double": ("\u2551   ", "\u2560\u2550\u2550 ", "\u255a\u2550\u2550 "),
-    "custom": ("", "", ""),
-}
+T = TypeVar("T", bound=Node)
 
 
 def print_tree(
-    tree: Node,
+    tree: T,
     node_name_or_path: str = "",
     max_depth: int = 0,
     all_attrs: bool = False,
-    attr_list: List[str] = [],
+    attr_list: Iterable[str] = [],
     attr_omit_null: bool = False,
     attr_bracket: List[str] = ["[", "]"],
     style: str = "const",
-    custom_style: List[str] = [],
+    custom_style: Iterable[str] = [],
 ) -> None:
     """Print tree to console, starting from `tree`.
 
@@ -143,11 +159,11 @@ def print_tree(
         node_name_or_path (str): node to print from, becomes the root node of printing
         max_depth (int): maximum depth of tree to print, based on `depth` attribute, optional
         all_attrs (bool): indicator to show all attributes, defaults to False, overrides `attr_list` and `attr_omit_null`
-        attr_list (List[str]): list of node attributes to print, optional
+        attr_list (Iterable[str]): list of node attributes to print, optional
         attr_omit_null (bool): indicator whether to omit showing of null attributes, defaults to False
         attr_bracket (List[str]): open and close bracket for `all_attrs` or `attr_list`
         style (str): style of print, defaults to abstract style
-        custom_style (List[str]): style of stem, branch and final stem, used when `style` is set to 'custom'
+        custom_style (Iterable[str]): style of stem, branch and final stem, used when `style` is set to 'custom'
     """
     for pre_str, fill_str, _node in yield_tree(
         tree=tree,
@@ -188,12 +204,12 @@ def print_tree(
 
 
 def yield_tree(
-    tree: Node,
+    tree: T,
     node_name_or_path: str = "",
     max_depth: int = 0,
     style: str = "const",
-    custom_style: List[str] = [],
-) -> Iterable[Tuple[str, str, Node]]:
+    custom_style: Iterable[str] = [],
+) -> Iterable[Tuple[str, str, T]]:
     """Generator method for customizing printing of tree, starting from `tree`.
 
     - Able to select which node to print from, resulting in a subtree, using `node_name_or_path`
@@ -298,8 +314,9 @@ def yield_tree(
         node_name_or_path (str): node to print from, becomes the root node of printing, optional
         max_depth (int): maximum depth of tree to print, based on `depth` attribute, optional
         style (str): style of print, defaults to abstract style
-        custom_style (List[str]): style of stem, branch and final stem, used when `style` is set to 'custom'
+        custom_style (Iterable[str]): style of stem, branch and final stem, used when `style` is set to 'custom'
     """
+    available_styles = ExportConstants.AVAILABLE_STYLES
     if style not in available_styles.keys():
         raise ValueError(
             f"Choose one of {available_styles.keys()} style, use `custom` to define own style"
@@ -315,7 +332,7 @@ def yield_tree(
 
     # Set style
     if style == "custom":
-        if len(custom_style) != 3:
+        if len(list(custom_style)) != 3:
             raise ValueError(
                 "Custom style selected, please specify the style of stem, branch, and final stem in `custom_style`"
             )
@@ -358,7 +375,7 @@ def yield_tree(
 
 
 def tree_to_dict(
-    tree: Node,
+    tree: T,
     name_key: str = "name",
     parent_key: str = "",
     attr_dict: Dict[str, str] = {},
@@ -393,9 +410,9 @@ def tree_to_dict(
         parent_key (str): dictionary key for `node.parent.node_name`, optional
         attr_dict (Dict[str, str]): dictionary mapping node attributes to dictionary key,
             key: node attributes, value: corresponding dictionary key, optional
-        all_attrs (bool): indicator whether to retrieve all `Node` attributes, defaults to False
+        all_attrs (bool): indicator whether to retrieve all ``Node`` attributes, overrides `attr_dict`, defaults to False
         max_depth (int): maximum depth to export tree, optional
-        skip_depth (int): number of initial depth to skip, optional
+        skip_depth (int): number of initial depths to skip, optional
         leaf_only (bool): indicator to retrieve only information from leaf nodes
 
     Returns:
@@ -404,7 +421,7 @@ def tree_to_dict(
     tree = tree.copy()
     data_dict = {}
 
-    def recursive_append(node: Node) -> None:
+    def recursive_append(node: T) -> None:
         if node:
             if (
                 (not max_depth or node.depth <= max_depth)
@@ -439,7 +456,7 @@ def tree_to_dict(
 
 
 def tree_to_nested_dict(
-    tree: Node,
+    tree: T,
     name_key: str = "name",
     child_key: str = "children",
     attr_dict: Dict[str, str] = {},
@@ -467,7 +484,7 @@ def tree_to_nested_dict(
         child_key (str): dictionary key for list of children, optional
         attr_dict (Dict[str, str]): dictionary mapping node attributes to dictionary key,
             key: node attributes, value: corresponding dictionary key, optional
-        all_attrs (bool): indicator whether to retrieve all `Node` attributes, defaults to False
+        all_attrs (bool): indicator whether to retrieve all ``Node`` attributes, overrides `attr_dict`, defaults to False
         max_depth (int): maximum depth to export tree, optional
 
     Returns:
@@ -476,7 +493,7 @@ def tree_to_nested_dict(
     tree = tree.copy()
     data_dict: Dict[str, List[Dict[str, Any]]] = {}
 
-    def recursive_append(node: Node, parent_dict: Dict[str, Any]) -> None:
+    def recursive_append(node: T, parent_dict: Dict[str, Any]) -> None:
         if node:
             if not max_depth or node.depth <= max_depth:
                 data_child = {name_key: node.node_name}
@@ -503,8 +520,9 @@ def tree_to_nested_dict(
     return data_dict[child_key][0]
 
 
+@optional_dependencies_pandas
 def tree_to_dataframe(
-    tree: Node,
+    tree: T,
     path_col: str = "path",
     name_col: str = "name",
     parent_col: str = "",
@@ -543,14 +561,14 @@ def tree_to_dataframe(
 
     Args:
         tree (Node): tree to be exported
-        path_col (str): column name for `node.path_name`, optional
+        path_col (str): column name for `node.path_name`, defaults to 'path'
         name_col (str): column name for `node.node_name`, defaults to 'name'
         parent_col (str): column name for `node.parent.node_name`, optional
         attr_dict (Dict[str, str]): dictionary mapping node attributes to column name,
             key: node attributes, value: corresponding column in dataframe, optional
-        all_attrs (bool): indicator whether to retrieve all `Node` attributes, defaults to False
+        all_attrs (bool): indicator whether to retrieve all ``Node`` attributes, overrides `attr_dict`, defaults to False
         max_depth (int): maximum depth to export tree, optional
-        skip_depth (int): number of initial depth to skip, optional
+        skip_depth (int): number of initial depths to skip, optional
         leaf_only (bool): indicator to retrieve only information from leaf nodes
 
     Returns:
@@ -559,7 +577,7 @@ def tree_to_dataframe(
     tree = tree.copy()
     data_list = []
 
-    def recursive_append(node: Node) -> None:
+    def recursive_append(node: T) -> None:
         if node:
             if (
                 (not max_depth or node.depth <= max_depth)
@@ -592,19 +610,20 @@ def tree_to_dataframe(
     return pd.DataFrame(data_list)
 
 
-def tree_to_dot(  # type: ignore[no-untyped-def]
-    tree: Union[Node, List[Node]],
+@optional_dependencies_image("pydot")
+def tree_to_dot(
+    tree: Union[T, List[T]],
     directed: bool = True,
     rankdir: str = "TB",
     bg_colour: str = "",
     node_colour: str = "",
     node_shape: str = "",
     edge_colour: str = "",
-    node_attr: str = "",
-    edge_attr: str = "",
-):
+    node_attr: Callable[[T], Dict[str, Any]] | str = "",
+    edge_attr: Callable[[T], Dict[str, Any]] | str = "",
+) -> pydot.Dot:
     r"""Export tree or list of trees to image.
-    Posible node attributes include style, fillcolor, shape.
+    Possible node attributes include style, fillcolor, shape.
 
     >>> from bigtree import Node, tree_to_dot
     >>> root = Node("a", age=90)
@@ -614,6 +633,12 @@ def tree_to_dot(  # type: ignore[no-untyped-def]
     >>> e = Node("e", age=35, parent=b)
     >>> graph = tree_to_dot(root)
 
+    Display image directly without saving (requires IPython)
+
+    >>> from IPython.display import Image, display
+    >>> plt = Image(graph.create_png())
+    >>> display(plt)
+
     Export to image, dot file, etc.
 
     >>> graph.write_png("tree.png")
@@ -624,7 +649,7 @@ def tree_to_dot(  # type: ignore[no-untyped-def]
     >>> graph.to_string()
     'strict digraph G {\nrankdir=TB;\na0 [label=a];\nb0 [label=b];\na0 -> b0;\nd0 [label=d];\nb0 -> d0;\ne0 [label=e];\nb0 -> e0;\nc0 [label=c];\na0 -> c0;\n}\n'
 
-    Defining node and edge attributes
+    Defining node and edge attributes (using node attribute)
 
     >>> class CustomNode(Node):
     ...     def __init__(self, name, node_shape="", edge_label="", **kwargs):
@@ -655,72 +680,71 @@ def tree_to_dot(  # type: ignore[no-untyped-def]
 
     .. image:: https://github.com/kayjan/bigtree/raw/master/assets/custom_tree.png
 
+    Alternative way to define node and edge attributes (using callable function)
+
+    >>> def get_node_attribute(node: Node):
+    ...     if node.is_leaf:
+    ...         return {"shape": "square"}
+    ...     return {"shape": "circle"}
+    >>>
+    >>>
+    >>> root = CustomNode("a")
+    >>> b = CustomNode("b", parent=root)
+    >>> c = CustomNode("c", parent=root)
+    >>> d = CustomNode("d", parent=b)
+    >>> e = CustomNode("e", parent=b)
+    >>> graph = tree_to_dot(root, node_colour="gold", node_attr=get_node_attribute)
+    >>> graph.write_png("assets/custom_tree_callable.png")
+
+    .. image:: https://github.com/kayjan/bigtree/raw/master/assets/custom_tree_callable.png
+
     Args:
         tree (Node/List[Node]): tree or list of trees to be exported
         directed (bool): indicator whether graph should be directed or undirected, defaults to True
-        rankdir (str): set direction of graph layout, defaults to 'TB' (top to bottom), can be 'BT' (bottom to top),
+        rankdir (str): layout direction, defaults to 'TB' (top to bottom), can be 'BT' (bottom to top),
             'LR' (left to right), 'RL' (right to left)
         bg_colour (str): background color of image, defaults to None
         node_colour (str): fill colour of nodes, defaults to None
         node_shape (str): shape of nodes, defaults to None
             Possible node_shape include "circle", "square", "diamond", "triangle"
         edge_colour (str): colour of edges, defaults to None
-        node_attr (str): `Node` attribute for node style, overrides `node_colour` and `node_shape`, defaults to None.
-            Possible node style (attribute value) include {"style": "filled", "fillcolor": "gold", "shape": "diamond"}
-        edge_attr (str): `Node` attribute for edge style, overrides `edge_colour`, defaults to None.
-            Possible edge style (attribute value) include {"style": "bold", "label": "edge label", "color": "black"}
+        node_attr (str | Callable): If string type, it refers to ``Node`` attribute for node style.
+            If callable type, it takes in the node itself and returns the node style.
+            This overrides `node_colour` and `node_shape` and defaults to None.
+            Possible node styles include {"style": "filled", "fillcolor": "gold", "shape": "diamond"}
+        edge_attr (str | Callable): If stirng type, it refers to ``Node`` attribute for edge style.
+            If callable type, it takes in the node itself and returns the edge style.
+            This overrides `edge_colour`, and defaults to None.
+            Possible edge styles include {"style": "bold", "label": "edge label", "color": "black"}
 
     Returns:
         (pydot.Dot)
     """
-    try:
-        import pydot
-    except ImportError:  # pragma: no cover
-        raise ImportError(
-            "pydot not available. Please perform a\n\npip install 'bigtree[image]'\n\nto install required dependencies"
-        )
-
     # Get style
-    if bg_colour:
-        graph_style = dict(bgcolor=bg_colour)
-    else:
-        graph_style = dict()
-
-    if node_colour:
-        node_style = dict(style="filled", fillcolor=node_colour)
-    else:
-        node_style = dict()
-
+    graph_style = dict(bgcolor=bg_colour) if bg_colour else dict()
+    node_style = dict(style="filled", fillcolor=node_colour) if node_colour else dict()
     if node_shape:
         node_style["shape"] = node_shape
-
-    if edge_colour:
-        edge_style = dict(color=edge_colour)
-    else:
-        edge_style = dict()
+    edge_style = dict(color=edge_colour) if edge_colour else dict()
 
     tree = tree.copy()
-
-    if directed:
-        _graph = pydot.Dot(
-            graph_type="digraph", strict=True, rankdir=rankdir, **graph_style
-        )
-    else:
-        _graph = pydot.Dot(
-            graph_type="graph", strict=True, rankdir=rankdir, **graph_style
-        )
+    _graph = (
+        pydot.Dot(graph_type="digraph", strict=True, rankdir=rankdir, **graph_style)
+        if directed
+        else pydot.Dot(graph_type="graph", strict=True, rankdir=rankdir, **graph_style)
+    )
 
     if not isinstance(tree, list):
         tree = [tree]
 
     for _tree in tree:
         if not isinstance(_tree, Node):
-            raise ValueError("Tree should be of type `Node`, or inherit from `Node`")
+            raise TypeError("Tree should be of type `Node`, or inherit from `Node`")
 
         name_dict: Dict[str, List[str]] = collections.defaultdict(list)
 
         def recursive_create_node_and_edges(
-            parent_name: Optional[str], child_node: Node
+            parent_name: Optional[str], child_node: T
         ) -> None:
             _node_style = node_style.copy()
             _edge_style = edge_style.copy()
@@ -731,10 +755,16 @@ def tree_to_dot(  # type: ignore[no-untyped-def]
             child_name = child_label + str(
                 name_dict[child_label].index(child_node.path_name)
             )
-            if node_attr and child_node.get_attr(node_attr):
-                _node_style.update(child_node.get_attr(node_attr))
+            if node_attr:
+                if isinstance(node_attr, str) and child_node.get_attr(node_attr):
+                    _node_style.update(child_node.get_attr(node_attr))
+                elif isinstance(node_attr, Callable):  # type: ignore
+                    _node_style.update(node_attr(child_node))  # type: ignore
             if edge_attr:
-                _edge_style.update(child_node.get_attr(edge_attr))
+                if isinstance(edge_attr, str) and child_node.get_attr(edge_attr):
+                    _edge_style.update(child_node.get_attr(edge_attr))
+                elif isinstance(edge_attr, Callable):  # type: ignore
+                    _edge_style.update(edge_attr(child_node))  # type: ignore
             node = pydot.Node(name=child_name, label=child_label, **_node_style)
             _graph.add_node(node)
             if parent_name is not None:
@@ -748,17 +778,18 @@ def tree_to_dot(  # type: ignore[no-untyped-def]
     return _graph
 
 
-def tree_to_pillow(  # type: ignore[no-untyped-def]
-    tree: Node,
+@optional_dependencies_image("Pillow")
+def tree_to_pillow(
+    tree: T,
     width: int = 0,
     height: int = 0,
     start_pos: Tuple[int, int] = (10, 10),
-    font_family: str = "assets/DejaVuSans.ttf",
+    font_family: str = "",
     font_size: int = 12,
     font_colour: Union[Tuple[int, int, int], str] = "black",
     bg_colour: Union[Tuple[int, int, int], str] = "white",
     **kwargs: Any,
-):
+) -> Image.Image:
     """Export tree to image (JPG, PNG).
     Image will be similar format as `print_tree`, accepts additional keyword arguments as input to `yield_tree`
 
@@ -788,15 +819,16 @@ def tree_to_pillow(  # type: ignore[no-untyped-def]
     Returns:
         (PIL.Image.Image)
     """
-    try:
-        from PIL import Image, ImageDraw, ImageFont
-    except ImportError:  # pragma: no cover
-        raise ImportError(
-            "Pillow not available. Please perform a\n\npip install 'bigtree[image]'\n\nto install required dependencies"
-        )
-
     # Initialize font
-    font = ImageFont.truetype(font_family, font_size)
+    if not font_family:
+        dejavusans_url = "https://github.com/kayjan/bigtree/raw/master/assets/DejaVuSans.ttf?raw=true"
+        font_family = urlopen(dejavusans_url)
+    try:
+        font = ImageFont.truetype(font_family, font_size)
+    except OSError:
+        raise ValueError(
+            f"Font file {font_family} is not found, set `font_family` parameter to point to a valid .ttf file."
+        )
 
     # Initialize text
     image_text = []
@@ -835,3 +867,291 @@ def tree_to_pillow(  # type: ignore[no-untyped-def]
     image_draw = ImageDraw.Draw(image)
     image_draw.text(start_pos, image_text_str, font=font, fill=font_colour)
     return image
+
+
+def tree_to_mermaid(
+    tree: T,
+    title: str = "",
+    rankdir: str = "TB",
+    line_shape: str = "basis",
+    node_colour: str = "",
+    node_border_colour: str = "",
+    node_border_width: float = 1,
+    node_shape: str = "rounded_edge",
+    node_shape_attr: Callable[[T], str] | str = "",
+    edge_arrow: str = "normal",
+    edge_arrow_attr: Callable[[T], str] | str = "",
+    edge_label: str = "",
+    node_attr: Callable[[T], str] | str = "",
+    **kwargs: Any,
+) -> str:
+    r"""Export tree to mermaid Markdown text. Accepts additional keyword arguments as input to `yield_tree`
+
+    Parameters for customizations that applies to entire flowchart include
+        - Title, `title`
+        - Layout direction, `rankdir`
+        - Line shape or curvature, `line_shape`
+        - Fill colour of nodes, `node_colour`
+        - Border colour of nodes, `node_border_colour`
+        - Border width of nodes, `node_border_width`
+        - Node shape, `node_shape`
+        - Edge arrow style, `edge_arrow`
+
+    Parameters for customizations that apply to customized nodes
+        - Fill colour of nodes, fill under `node_attr`
+        - Border colour of nodes, stroke under `node_attr`
+        - Border width of nodes, stroke-width under `node_attr`
+        - Node shape, `node_shape_attr`
+        - Edge arrow style, `edge_arrow_attr`
+        - Edge label, `edge_label`
+
+    **Accepted Parameter Values**
+
+    Possible `rankdir`
+        - `TB`: top-to-bottom
+        - `BT`: bottom-to-top
+        - `LR`: left-to-right
+        - `RL`: right-to-left
+
+    Possible `line_shape`
+        - `basis`
+        - `bumpX`: used in LR or RL direction
+        - `bumpY`
+        - `cardinal`: undirected
+        - `catmullRom`: undirected
+        - `linear`:
+        - `monotoneX`: used in LR or RL direction
+        - `monotoneY`
+        - `natural`
+        - `step`: used in LR or RL direction
+        - `stepAfter`
+        - `stepBefore`: used in LR or RL direction
+
+    Possible `node_shape`
+        - `rounded_edge`: rectangular with rounded edges
+        - `stadium`: (_) shape, rectangular with rounded ends
+        - `subroutine`: ||_|| shape, rectangular with additional line at the ends
+        - `cylindrical`: database node
+        - `circle`: circular
+        - `asymmetric`: >_| shape
+        - `rhombus`: decision node
+        - `hexagon`: <_> shape
+        - `parallelogram`: /_/ shape
+        - `parallelogram_alt`: \\_\\ shape, inverted parallelogram
+        - `trapezoid`: /_\\ shape
+        - `trapezoid_alt`: \\_/ shape, inverted trapezoid
+        - `double_circle`
+
+    Possible `edge_arrow`
+        - `normal`: directed arrow, shaded arrowhead
+        - `bold`: bold directed arrow
+        - `dotted`: dotted directed arrow
+        - `open`: line, undirected arrow
+        - `bold_open`: bold line
+        - `dotted_open`: dotted line
+        - `invisible`: no line
+        - `circle`: directed arrow with filled circle arrowhead
+        - `cross`: directed arrow with cross arrowhead
+        - `double_normal`: bidirectional directed arrow
+        - `double_circle`: bidirectional directed arrow with filled circle arrowhead
+        - `double_cross`: bidirectional directed arrow with cross arrowhead
+
+    Refer to mermaid `documentation`_ for more information.
+    Paste the output into any markdown file renderer to view the flowchart, alternatively visit the
+    mermaid playground `here`_.
+
+    .. note:: Advanced mermaid flowchart functionalities such as subgraphs and interactions (script, click) are not supported.
+
+    .. _documentation: http://mermaid.js.org/syntax/flowchart.html
+    .. _here: https://mermaid.live/
+
+    >>> from bigtree import tree_to_mermaid
+    >>> root = Node("a", node_shape="rhombus")
+    >>> b = Node("b", edge_arrow="bold", edge_label="Child 1", parent=root)
+    >>> c = Node("c", edge_arrow="dotted", edge_label="Child 2", parent=root)
+    >>> d = Node("d", node_style="fill:yellow, stroke:black", parent=b)
+    >>> e = Node("e", parent=b)
+    >>> graph = tree_to_mermaid(root)
+    >>> print(graph)
+    ```mermaid
+    %%{ init: { 'flowchart': { 'curve': 'basis' } } }%%
+    flowchart TB
+    0("a") --> 0-0("b")
+    0-0 --> 0-0-0("d")
+    0-0 --> 0-0-1("e")
+    0("a") --> 0-1("c")
+    classDef default stroke-width:1
+    ```
+
+    **Customize node shape, edge label, edge arrow, and custom node attributes**
+
+    >>> graph = tree_to_mermaid(root, node_shape_attr="node_shape", edge_label="edge_label", edge_arrow_attr="edge_arrow", node_attr="node_style")
+    >>> print(graph)
+    ```mermaid
+    %%{ init: { 'flowchart': { 'curve': 'basis' } } }%%
+    flowchart TB
+    0{"a"} ==>|Child 1| 0-0("b")
+    0-0:::class0-0-0 --> 0-0-0("d")
+    0-0 --> 0-0-1("e")
+    0{"a"} -.->|Child 2| 0-1("c")
+    classDef default stroke-width:1
+    classDef class0-0-0 fill:yellow, stroke:black
+    ```
+
+    Args:
+        tree (Node): tree to be exported
+        title (str): title, defaults to None
+        rankdir (str): layout direction, defaults to 'TB' (top to bottom), can be 'BT' (bottom to top),
+            'LR' (left to right), 'RL' (right to left)
+        line_shape (str): line shape or curvature, defaults to 'basis'
+        node_colour (str): fill colour of nodes, can be colour name or hexcode, defaults to None
+        node_border_colour (str): border colour of nodes, can be colour name or hexcode, defaults to None
+        node_border_width (float): width of node border, defaults to 1
+        node_shape (str): node shape, sets the shape of every node, defaults to 'rounded_edge'
+        node_shape_attr (str | Callable): If string type, it refers to ``Node`` attribute for node shape.
+            If callable type, it takes in the node itself and returns the node shape.
+            This sets the shape of custom nodes, and overrides default `node_shape`, defaults to None
+        edge_arrow (str): edge arrow style from parent to itself, sets the arrow style of every edge, defaults to 'normal'
+        edge_arrow_attr (str | Callable): If string type, it refers to ``Node`` attribute for edge arrow style.
+            If callable type, it takes in the node itself and returns the edge arrow style.
+            This sets the edge arrow style of custom nodes from parent to itself, and overrides default `edge_arrow`, defaults to None
+        edge_label (str): ``Node`` attribute for edge label from parent to itself, defaults to None
+        node_attr (str | Callable): If string type, it refers to ``Node`` attribute for node style.
+            If callable type, it takes in the node itself and returns the node style.
+            This overrides `node_colour`, `node_border_colour`, and `node_border_width`, defaults to None
+
+    Returns:
+        (str)
+    """
+    from bigtree.tree.helper import clone_tree
+
+    rankdirs = MermaidConstants.RANK_DIR
+    line_shapes = MermaidConstants.LINE_SHAPES
+    node_shapes = MermaidConstants.NODE_SHAPES
+    edge_arrows = MermaidConstants.EDGE_ARROWS
+
+    # Assertions
+    assert_str_in_list("rankdir", rankdir, rankdirs)
+    assert_key_in_dict("node_shape", node_shape, node_shapes)
+    assert_str_in_list("line_shape", line_shape, line_shapes)
+    assert_key_in_dict("edge_arrow", edge_arrow, edge_arrows)
+
+    mermaid_template = """```mermaid\n{title}{line_style}\nflowchart {rankdir}\n{flows}\n{styles}\n```"""
+    flowchart_template = "{from_node_ref}{from_node_name}{flow_style} {arrow}{arrow_label} {to_node_ref}{to_node_name}"
+    style_template = "classDef {style_name} {style}"
+
+    # Content
+    title = f"---\ntitle: {title}\n---" if title else ""
+    line_style = f"%%{{ init: {{ 'flowchart': {{ 'curve': '{line_shape}' }} }} }}%%"
+    styles = []
+    flows = []
+
+    def _construct_style(
+        _style_name: str,
+        _node_colour: str,
+        _node_border_colour: str,
+        _node_border_width: float,
+    ) -> str:
+        style = []
+        if _node_colour:
+            style.append(f"fill:{_node_colour}")
+        if _node_border_colour:
+            style.append(f"stroke:{_node_border_colour}")
+        if _node_border_width:
+            style.append(f"stroke-width:{_node_border_width}")
+        if not style:
+            raise ValueError("Unable to construct style!")
+        return style_template.format(style_name=_style_name, style=",".join(style))
+
+    default_style = _construct_style(
+        "default", node_colour, node_border_colour, node_border_width
+    )
+    styles.append(default_style)
+
+    class MermaidNode(Node):
+        @property
+        def mermaid_name(self) -> str:
+            """Reference name for MermaidNode, must be unique for each node
+
+            Returns:
+                (str)
+            """
+            if self.is_root:
+                return "0"
+            return f"{self.parent.mermaid_name}-{self.parent.children.index(self)}"
+
+    def get_attr(
+        _node: MermaidNode,
+        attr_parameter: str | Callable[[MermaidNode], str],
+        default_parameter: str,
+    ) -> str:
+        """Get custom attribute if available, otherwise return default parameter
+
+        Args:
+            _node (MermaidNode): node to get custom attribute, can be accessed as node attribute or a callable that takes in the node
+            attr_parameter (str | Callable): custom attribute parameter
+            default_parameter (str): default parameter if there is no attr_parameter
+
+        Returns:
+            (str)
+        """
+        _choice = default_parameter
+        if attr_parameter:
+            if isinstance(attr_parameter, str):
+                _choice = _node.get_attr(attr_parameter, default_parameter)
+            else:
+                _choice = attr_parameter(_node)
+        return _choice
+
+    tree_mermaid = clone_tree(tree, MermaidNode)
+    for _, _, node in yield_tree(tree_mermaid, **kwargs):
+        if not node.is_root:
+            # Get custom style (node_shape_attr)
+            _parent_node_name = ""
+            if node.parent.is_root:
+                _parent_node_shape_choice = get_attr(
+                    node.parent, node_shape_attr, node_shape  # type: ignore
+                )
+                _parent_node_shape = node_shapes[_parent_node_shape_choice]
+                _parent_node_name = _parent_node_shape.format(label=node.parent.name)
+            _node_shape_choice = get_attr(node, node_shape_attr, node_shape)  # type: ignore
+            _node_shape = node_shapes[_node_shape_choice]
+            _node_name = _node_shape.format(label=node.name)
+
+            # Get custom style (edge_arrow_attr, edge_label)
+            _arrow_choice = get_attr(node, edge_arrow_attr, edge_arrow)  # type: ignore
+            _arrow = edge_arrows[_arrow_choice]
+            _arrow_label = (
+                f"|{node.get_attr(edge_label)}|" if node.get_attr(edge_label) else ""
+            )
+
+            # Get custom style (node_attr)
+            _flow_style = get_attr(node, node_attr, "")  # type: ignore
+            if _flow_style:
+                _flow_style_class = f"""class{node.get_attr("mermaid_name")}"""
+                styles.append(
+                    style_template.format(
+                        style_name=_flow_style_class, style=_flow_style
+                    )
+                )
+                _flow_style = f":::{_flow_style_class}"
+
+            flows.append(
+                flowchart_template.format(
+                    from_node_ref=node.parent.get_attr("mermaid_name"),
+                    from_node_name=_parent_node_name,
+                    flow_style=_flow_style,
+                    arrow=_arrow,
+                    arrow_label=_arrow_label,
+                    to_node_ref=node.get_attr("mermaid_name"),
+                    to_node_name=_node_name,
+                )
+            )
+
+    return mermaid_template.format(
+        title=title,
+        line_style=line_style,
+        rankdir=rankdir,
+        flows="\n".join(flows),
+        styles="\n".join(styles),
+    )
