@@ -2,7 +2,8 @@ import logging
 from typing import List, Optional
 
 from bigtree.node.node import Node
-from bigtree.tree.search import find_path
+from bigtree.tree.construct import add_path_to_tree
+from bigtree.tree.search import find_full_path, find_path
 from bigtree.utils.exceptions import NotFoundError, TreeError
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
@@ -10,8 +11,11 @@ logging.getLogger(__name__).addHandler(logging.NullHandler())
 __all__ = [
     "shift_nodes",
     "copy_nodes",
+    "shift_and_replace_nodes",
     "copy_nodes_from_tree_to_tree",
+    "copy_and_replace_nodes_from_tree_to_tree",
     "copy_or_shift_logic",
+    "replace_logic",
 ]
 
 
@@ -25,6 +29,7 @@ def shift_nodes(
     merge_children: bool = False,
     merge_leaves: bool = False,
     delete_children: bool = False,
+    with_full_path: bool = False,
 ) -> None:
     """Shift nodes from `from_paths` to `to_paths` *in-place*.
 
@@ -37,10 +42,14 @@ def shift_nodes(
 
     For paths in `from_paths` and `to_paths`,
       - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
       - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
       - Path name must be unique to one node.
 
     For paths in `to_paths`,
+      - Path name must be full path.
       - Can set to empty string or None to delete the path in `from_paths`, note that ``copy`` must be set to False.
 
     If ``merge_children=True``,
@@ -54,19 +63,19 @@ def shift_nodes(
       - If `to_path` is present and ``overriding=True``, it behaves like overriding and only new leaves are retained,
         original node in `from_path` is retained.
 
-    >>> from bigtree import Node, shift_nodes, print_tree
+    >>> from bigtree import Node, shift_nodes
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> c = Node("c", parent=root)
     >>> d = Node("d", parent=root)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     ├── c
     └── d
 
     >>> shift_nodes(root, ["a/c", "a/d"], ["a/b/c", "a/dummy/d"])
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -78,13 +87,13 @@ def shift_nodes(
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> c = Node("c", parent=root)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     └── c
 
     >>> shift_nodes(root, ["a/b"], [None])
-    >>> print_tree(root)
+    >>> root.show()
     a
     └── c
 
@@ -96,7 +105,7 @@ def shift_nodes(
     >>> d = Node("d", parent=c)
     >>> c2 = Node("c", parent=b)
     >>> e = Node("e", parent=c2)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -105,13 +114,13 @@ def shift_nodes(
         └── d
 
     >>> shift_nodes(root, ["a/b/c"], ["a/c"], overriding=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     └── c
         └── e
 
-    In ``merge_children`` case, child nodes are shifted instead of the parent node.
+    In ``merge_children=True`` case, child nodes are shifted instead of the parent node.
      - If the path already exists, child nodes are merged with existing children.
      - If same node is shifted, the child nodes of the node are merged with the node's parent.
 
@@ -126,7 +135,7 @@ def shift_nodes(
     >>> f = Node("f", parent=root)
     >>> g = Node("g", parent=f)
     >>> h = Node("h", parent=g)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -140,7 +149,7 @@ def shift_nodes(
             └── h
 
     >>> shift_nodes(root, ["a/b/c", "z", "a/f"], ["a/c", "a/z", "a/f"], merge_children=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     ├── c
@@ -150,7 +159,7 @@ def shift_nodes(
     └── g
         └── h
 
-    In ``merge_leaves`` case, leaf nodes are copied instead of the parent node.
+    In ``merge_leaves=True`` case, leaf nodes are copied instead of the parent node.
      - If the path already exists, leaf nodes are merged with existing children.
      - If same node is copied, the leaf nodes of the node are merged with the node's parent.
 
@@ -165,7 +174,7 @@ def shift_nodes(
     >>> f = Node("f", parent=root)
     >>> g = Node("g", parent=f)
     >>> h = Node("h", parent=g)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -179,7 +188,7 @@ def shift_nodes(
             └── h
 
     >>> shift_nodes(root, ["a/b/c", "z", "a/f"], ["a/c", "a/z", "a/f"], merge_leaves=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -192,7 +201,7 @@ def shift_nodes(
     ├── y
     └── h
 
-    In ``delete_children`` case, only the node is shifted without its accompanying children/descendants.
+    In ``delete_children=True`` case, only the node is shifted without its accompanying children/descendants.
 
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
@@ -202,7 +211,7 @@ def shift_nodes(
     >>> e = Node("e", parent=c2)
     >>> z = Node("z", parent=b)
     >>> y = Node("y", parent=z)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -213,7 +222,7 @@ def shift_nodes(
         └── d
 
     >>> shift_nodes(root, ["a/b/z"], ["a/z"], delete_children=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -232,6 +241,8 @@ def shift_nodes(
         merge_children (bool): indicator to merge children and remove intermediate parent node, defaults to False
         merge_leaves (bool): indicator to merge leaf nodes and remove intermediate parent node(s), defaults to False
         delete_children (bool): indicator to shift node only without children, defaults to False
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
     """
     return copy_or_shift_logic(
         tree=tree,
@@ -245,6 +256,7 @@ def shift_nodes(
         merge_leaves=merge_leaves,
         delete_children=delete_children,
         to_tree=None,
+        with_full_path=with_full_path,
     )  # pragma: no cover
 
 
@@ -258,6 +270,7 @@ def copy_nodes(
     merge_children: bool = False,
     merge_leaves: bool = False,
     delete_children: bool = False,
+    with_full_path: bool = False,
 ) -> None:
     """Copy nodes from `from_paths` to `to_paths` *in-place*.
 
@@ -270,9 +283,15 @@ def copy_nodes(
 
     For paths in `from_paths` and `to_paths`,
       - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
       - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
       - Path name must be unique to one node.
 
+    For paths in `to_paths`,
+      - Path name must be full path.
+
     If ``merge_children=True``,
       - If `to_path` is not present, it copies children of `from_path`.
       - If `to_path` is present, and ``overriding=False``, original and new children are merged.
@@ -283,19 +302,19 @@ def copy_nodes(
       - If `to_path` is present, and ``overriding=False``, original children and leaves are merged.
       - If `to_path` is present and ``overriding=True``, it behaves like overriding and only new leaves are retained.
 
-    >>> from bigtree import Node, copy_nodes, print_tree
+    >>> from bigtree import Node, copy_nodes
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> c = Node("c", parent=root)
     >>> d = Node("d", parent=root)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     ├── c
     └── d
 
     >>> copy_nodes(root, ["a/c", "a/d"], ["a/b/c", "a/dummy/d"])
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -312,7 +331,7 @@ def copy_nodes(
     >>> d = Node("d", parent=c)
     >>> c2 = Node("c", parent=b)
     >>> e = Node("e", parent=c2)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -321,7 +340,7 @@ def copy_nodes(
         └── d
 
     >>> copy_nodes(root, ["a/b/c"], ["a/c"], overriding=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   └── c
@@ -329,7 +348,7 @@ def copy_nodes(
     └── c
         └── e
 
-    In ``merge_children`` case, child nodes are copied instead of the parent node.
+    In ``merge_children=True`` case, child nodes are copied instead of the parent node.
      - If the path already exists, child nodes are merged with existing children.
      - If same node is copied, the child nodes of the node are merged with the node's parent.
 
@@ -344,7 +363,7 @@ def copy_nodes(
     >>> f = Node("f", parent=root)
     >>> g = Node("g", parent=f)
     >>> h = Node("h", parent=g)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -358,7 +377,7 @@ def copy_nodes(
             └── h
 
     >>> copy_nodes(root, ["a/b/c", "z", "a/f"], ["a/c", "a/z", "a/f"], merge_children=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -372,7 +391,7 @@ def copy_nodes(
     └── g
         └── h
 
-    In ``merge_leaves`` case, leaf nodes are copied instead of the parent node.
+    In ``merge_leaves=True`` case, leaf nodes are copied instead of the parent node.
      - If the path already exists, leaf nodes are merged with existing children.
      - If same node is copied, the leaf nodes of the node are merged with the node's parent.
 
@@ -387,7 +406,7 @@ def copy_nodes(
     >>> f = Node("f", parent=root)
     >>> g = Node("g", parent=f)
     >>> h = Node("h", parent=g)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -401,7 +420,7 @@ def copy_nodes(
             └── h
 
     >>> copy_nodes(root, ["a/b/c", "z", "a/f"], ["a/c", "a/z", "a/f"], merge_leaves=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -417,7 +436,7 @@ def copy_nodes(
     ├── y
     └── h
 
-    In ``delete_children`` case, only the node is copied without its accompanying children/descendants.
+    In ``delete_children=True`` case, only the node is copied without its accompanying children/descendants.
 
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
@@ -427,7 +446,7 @@ def copy_nodes(
     >>> e = Node("e", parent=c2)
     >>> z = Node("z", parent=b)
     >>> y = Node("y", parent=z)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -438,7 +457,7 @@ def copy_nodes(
         └── d
 
     >>> copy_nodes(root, ["a/b/z"], ["a/z"], delete_children=True)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── c
@@ -459,6 +478,8 @@ def copy_nodes(
         merge_children (bool): indicator to merge children and remove intermediate parent node, defaults to False
         merge_leaves (bool): indicator to merge leaf nodes and remove intermediate parent node(s), defaults to False
         delete_children (bool): indicator to copy node only without children, defaults to False
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
     """
     return copy_or_shift_logic(
         tree=tree,
@@ -472,6 +493,98 @@ def copy_nodes(
         merge_leaves=merge_leaves,
         delete_children=delete_children,
         to_tree=None,
+        with_full_path=with_full_path,
+    )  # pragma: no cover
+
+
+def shift_and_replace_nodes(
+    tree: Node,
+    from_paths: List[str],
+    to_paths: List[str],
+    sep: str = "/",
+    skippable: bool = False,
+    delete_children: bool = False,
+    with_full_path: bool = False,
+) -> None:
+    """Shift nodes from `from_paths` to *replace* `to_paths` *in-place*.
+
+    - Creates intermediate nodes if to path is not present
+    - Able to skip nodes if from path is not found, defaults to False (from-nodes must be found; not skippable).
+    - Able to shift node only and delete children, defaults to False (nodes are shifted together with children).
+
+    For paths in `from_paths` and `to_paths`,
+      - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
+      - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
+      - Path name must be unique to one node.
+
+    For paths in `to_paths`,
+      - Path name must be full path.
+      - Path must exist, node-to-be-replaced must be present.
+
+    >>> from bigtree import Node, shift_and_replace_nodes
+    >>> root = Node("a")
+    >>> b = Node("b", parent=root)
+    >>> c = Node("c", parent=b)
+    >>> d = Node("d", parent=root)
+    >>> e = Node("e", parent=d)
+    >>> root.show()
+    a
+    ├── b
+    │   └── c
+    └── d
+        └── e
+
+    >>> shift_and_replace_nodes(root, ["a/b"], ["a/d/e"])
+    >>> root.show()
+    a
+    └── d
+        └── b
+            └── c
+
+    In ``delete_children=True`` case, only the node is shifted without its accompanying children/descendants.
+
+    >>> from bigtree import Node, shift_and_replace_nodes
+    >>> root = Node("a")
+    >>> b = Node("b", parent=root)
+    >>> c = Node("c", parent=b)
+    >>> d = Node("d", parent=root)
+    >>> e = Node("e", parent=d)
+    >>> root.show()
+    a
+    ├── b
+    │   └── c
+    └── d
+        └── e
+
+    >>> shift_and_replace_nodes(root, ["a/b"], ["a/d/e"], delete_children=True)
+    >>> root.show()
+    a
+    └── d
+        └── b
+
+    Args:
+        tree (Node): tree to modify
+        from_paths (List[str]): original paths to shift nodes from
+        to_paths (List[str]): new paths to shift nodes to
+        sep (str): path separator for input paths, applies to `from_path` and `to_path`
+        skippable (bool): indicator to skip if from path is not found, defaults to False
+        delete_children (bool): indicator to shift node only without children, defaults to False
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
+    """
+    return replace_logic(
+        tree=tree,
+        from_paths=from_paths,
+        to_paths=to_paths,
+        sep=sep,
+        copy=False,
+        skippable=skippable,
+        delete_children=delete_children,
+        to_tree=None,
+        with_full_path=with_full_path,
     )  # pragma: no cover
 
 
@@ -486,6 +599,7 @@ def copy_nodes_from_tree_to_tree(
     merge_children: bool = False,
     merge_leaves: bool = False,
     delete_children: bool = False,
+    with_full_path: bool = False,
 ) -> None:
     """Copy nodes from `from_paths` to `to_paths` *in-place*.
 
@@ -498,9 +612,15 @@ def copy_nodes_from_tree_to_tree(
 
     For paths in `from_paths` and `to_paths`,
       - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
       - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
       - Path name must be unique to one node.
 
+    For paths in `to_paths`,
+      - Path name must be full path.
+
     If ``merge_children=True``,
       - If `to_path` is not present, it copies children of `from_path`
       - If `to_path` is present, and ``overriding=False``, original and new children are merged
@@ -511,7 +631,7 @@ def copy_nodes_from_tree_to_tree(
       - If `to_path` is present, and ``overriding=False``, original children and leaves are merged.
       - If `to_path` is present and ``overriding=True``, it behaves like overriding and only new leaves are retained.
 
-    >>> from bigtree import Node, copy_nodes_from_tree_to_tree, print_tree
+    >>> from bigtree import Node, copy_nodes_from_tree_to_tree
     >>> root = Node("a")
     >>> b = Node("b", parent=root)
     >>> c = Node("c", parent=root)
@@ -519,7 +639,7 @@ def copy_nodes_from_tree_to_tree(
     >>> e = Node("e", parent=root)
     >>> f = Node("f", parent=e)
     >>> g = Node("g", parent=f)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     ├── c
@@ -530,7 +650,7 @@ def copy_nodes_from_tree_to_tree(
 
     >>> root_other = Node("aa")
     >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/b", "a/c", "a/e"], ["aa/b", "aa/b/c", "aa/dummy/e"])
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
     ├── b
     │   └── c
@@ -545,31 +665,31 @@ def copy_nodes_from_tree_to_tree(
     >>> root_other = Node("aa")
     >>> c = Node("c", parent=root_other)
     >>> e = Node("e", parent=c)
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
     └── c
         └── e
 
     >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/b", "a/c"], ["aa/b", "aa/c"], overriding=True)
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
     ├── b
     └── c
         └── d
 
-    In ``merge_children`` case, child nodes are copied instead of the parent node.
+    In ``merge_children=True`` case, child nodes are copied instead of the parent node.
      - If the path already exists, child nodes are merged with existing children.
 
     >>> root_other = Node("aa")
     >>> c = Node("c", parent=root_other)
     >>> e = Node("e", parent=c)
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
     └── c
         └── e
 
-    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["a/c", "a/e"], merge_children=True)
-    >>> print_tree(root_other)
+    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["aa/c", "aa/e"], merge_children=True)
+    >>> root_other.show()
     aa
     ├── c
     │   ├── e
@@ -577,33 +697,33 @@ def copy_nodes_from_tree_to_tree(
     └── f
         └── g
 
-    In ``merge_leaves`` case, leaf nodes are copied instead of the parent node.
+    In ``merge_leaves=True`` case, leaf nodes are copied instead of the parent node.
      - If the path already exists, leaf nodes are merged with existing children.
 
     >>> root_other = Node("aa")
     >>> c = Node("c", parent=root_other)
     >>> e = Node("e", parent=c)
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
     └── c
         └── e
 
-    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["a/c", "a/e"], merge_leaves=True)
-    >>> print_tree(root_other)
+    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["aa/c", "aa/e"], merge_leaves=True)
+    >>> root_other.show()
     aa
     ├── c
     │   ├── e
     │   └── d
     └── g
 
-    In ``delete_children`` case, only the node is copied without its accompanying children/descendants.
+    In ``delete_children=True`` case, only the node is copied without its accompanying children/descendants.
 
     >>> root_other = Node("aa")
-    >>> print_tree(root_other)
+    >>> root_other.show()
     aa
 
-    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["a/c", "a/e"], delete_children=True)
-    >>> print_tree(root_other)
+    >>> copy_nodes_from_tree_to_tree(root, root_other, ["a/c", "e"], ["aa/c", "aa/e"], delete_children=True)
+    >>> root_other.show()
     aa
     ├── c
     └── e
@@ -619,6 +739,8 @@ def copy_nodes_from_tree_to_tree(
         merge_children (bool): indicator to merge children and remove intermediate parent node, defaults to False
         merge_leaves (bool): indicator to merge leaf nodes and remove intermediate parent node(s), defaults to False
         delete_children (bool): indicator to copy node only without children, defaults to False
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
     """
     return copy_or_shift_logic(
         tree=from_tree,
@@ -632,6 +754,115 @@ def copy_nodes_from_tree_to_tree(
         merge_leaves=merge_leaves,
         delete_children=delete_children,
         to_tree=to_tree,
+        with_full_path=with_full_path,
+    )  # pragma: no cover
+
+
+def copy_and_replace_nodes_from_tree_to_tree(
+    from_tree: Node,
+    to_tree: Node,
+    from_paths: List[str],
+    to_paths: List[str],
+    sep: str = "/",
+    skippable: bool = False,
+    delete_children: bool = False,
+    with_full_path: bool = False,
+) -> None:
+    """Copy nodes from `from_paths` to *replace* `to_paths` *in-place*.
+
+    - Creates intermediate nodes if to path is not present
+    - Able to skip nodes if from path is not found, defaults to False (from-nodes must be found; not skippable).
+    - Able to copy node only and delete children, defaults to False (nodes are copied together with children).
+
+    For paths in `from_paths` and `to_paths`,
+      - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
+      - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
+      - Path name must be unique to one node.
+
+    For paths in `to_paths`,
+      - Path name must be full path.
+      - Path must exist, node-to-be-replaced must be present.
+
+    >>> from bigtree import Node, copy_and_replace_nodes_from_tree_to_tree
+    >>> root = Node("a")
+    >>> b = Node("b", parent=root)
+    >>> c = Node("c", parent=root)
+    >>> d = Node("d", parent=c)
+    >>> e = Node("e", parent=root)
+    >>> f = Node("f", parent=e)
+    >>> g = Node("g", parent=f)
+    >>> root.show()
+    a
+    ├── b
+    ├── c
+    │   └── d
+    └── e
+        └── f
+            └── g
+
+    >>> root_other = Node("aa")
+    >>> bb = Node("bb", parent=root_other)
+    >>> cc = Node("cc", parent=bb)
+    >>> dd = Node("dd", parent=root_other)
+    >>> root_other.show()
+    aa
+    ├── bb
+    │   └── cc
+    └── dd
+
+    >>> copy_and_replace_nodes_from_tree_to_tree(root, root_other, ["a/c", "a/e"], ["aa/bb/cc", "aa/dd"])
+    >>> root_other.show()
+    aa
+    ├── bb
+    │   └── c
+    │       └── d
+    └── e
+        └── f
+            └── g
+
+    In ``delete_children=True`` case, only the node is copied without its accompanying children/descendants.
+
+    >>> root_other = Node("aa")
+    >>> bb = Node("bb", parent=root_other)
+    >>> cc = Node("cc", parent=bb)
+    >>> dd = Node("dd", parent=root_other)
+    >>> root_other.show()
+    aa
+    ├── bb
+    │   └── cc
+    └── dd
+
+    >>> copy_and_replace_nodes_from_tree_to_tree(root, root_other, ["a/c", "a/e"], ["aa/bb/cc", "aa/dd"], delete_children=True)
+    >>> root_other.show()
+    aa
+    ├── bb
+    │   └── c
+    └── e
+
+    Args:
+        from_tree (Node): tree to copy nodes from
+        to_tree (Node): tree to copy nodes to
+        from_paths (List[str]): original paths to shift nodes from
+        to_paths (List[str]): new paths to shift nodes to
+        sep (str): path separator for input paths, applies to `from_path` and `to_path`
+        skippable (bool): indicator to skip if from path is not found, defaults to False
+        delete_children (bool): indicator to copy node only without children, defaults to False
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
+    """
+    return replace_logic(
+        tree=from_tree,
+        from_paths=from_paths,
+        to_paths=to_paths,
+        sep=sep,
+        copy=True,
+        skippable=skippable,
+        delete_children=delete_children,
+        to_tree=to_tree,
+        with_full_path=with_full_path,
     )  # pragma: no cover
 
 
@@ -647,6 +878,7 @@ def copy_or_shift_logic(
     merge_leaves: bool = False,
     delete_children: bool = False,
     to_tree: Optional[Node] = None,
+    with_full_path: bool = False,
 ) -> None:
     """Shift or copy nodes from `from_paths` to `to_paths` *in-place*.
 
@@ -661,10 +893,14 @@ def copy_or_shift_logic(
 
     For paths in `from_paths` and `to_paths`,
       - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
       - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
       - Path name must be unique to one node.
 
     For paths in `to_paths`,
+      - Path name must be full path.
       - Can set to empty string or None to delete the path in `from_paths`, note that ``copy`` must be set to False.
 
     If ``merge_children=True``,
@@ -690,6 +926,8 @@ def copy_or_shift_logic(
         merge_leaves (bool): indicator to merge leaf nodes and remove intermediate parent node(s), defaults to False
         delete_children (bool): indicator to shift/copy node only without children, defaults to False
         to_tree (Node): tree to copy to, defaults to None
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
     """
     if merge_children and merge_leaves:
         raise ValueError(
@@ -704,25 +942,58 @@ def copy_or_shift_logic(
             f"Paths are different length, input `from_paths` have {len(from_paths)} entries, "
             f"while output `to_paths` have {len(to_paths)} entries"
         )
+    if copy and (None in to_paths or "" in to_paths):
+        raise ValueError(
+            "Deletion of node will not happen if `copy=True`, check your `copy` parameter."
+        )
+
+    # Modify `sep` of from_paths and to_paths
+    if not to_tree:
+        to_tree = tree
+    tree_sep = to_tree.sep
+    from_paths = [path.rstrip(sep).replace(sep, tree.sep) for path in from_paths]
+    to_paths = [
+        path.rstrip(sep).replace(sep, tree_sep) if path else None for path in to_paths
+    ]
+
     for from_path, to_path in zip(from_paths, to_paths):
         if to_path:
-            if from_path.split(sep)[-1] != to_path.split(sep)[-1]:
+            if from_path.split(tree.sep)[-1] != to_path.split(tree_sep)[-1]:
                 raise ValueError(
                     f"Unable to assign from_path {from_path} to to_path {to_path}\n"
                     f"Verify that `sep` is defined correctly for path\n"
-                    f"Alternatively, check that `from_path` and `to_path` is reassigning the same node"
+                    f"Alternatively, check that `from_path` and `to_path` is reassigning the same node."
                 )
 
-    transfer_indicator = False
-    node_type = tree.__class__
-    tree_sep = tree.sep
-    if to_tree:
-        transfer_indicator = True
-        node_type = to_tree.__class__
-        tree_sep = to_tree.sep
+    if with_full_path:
+        if not all(
+            [
+                path.lstrip(tree.sep).split(tree.sep)[0] == tree.root.node_name
+                for path in from_paths
+            ]
+        ):
+            raise ValueError(
+                "Invalid path in `from_paths` not starting with the root node. "
+                "Check your `from_paths` parameter, alternatively set `with_full_path=False` to shift "
+                "partial path instead of full path."
+            )
+    if not all(
+        [
+            path.lstrip(tree_sep).split(tree_sep)[0] == to_tree.root.node_name
+            for path in to_paths
+            if path
+        ]
+    ):
+        raise ValueError(
+            "Invalid path in `to_paths` not starting with the root node. Check your `to_paths` parameter."
+        )
+
+    # Perform shifting/copying
     for from_path, to_path in zip(from_paths, to_paths):
-        from_path = from_path.replace(sep, tree.sep)
-        from_node = find_path(tree, from_path)
+        if with_full_path:
+            from_node = find_full_path(tree, from_path)
+        else:
+            from_node = find_path(tree, from_path)
 
         # From node not found
         if not from_node:
@@ -741,11 +1012,7 @@ def copy_or_shift_logic(
                 to_node = None
             # Node to be copied/shifted
             else:
-                to_path = to_path.replace(sep, tree_sep)
-                if transfer_indicator:
-                    to_node = find_path(to_tree, to_path)
-                else:
-                    to_node = find_path(tree, to_path)
+                to_node = find_full_path(to_tree, to_path)
 
                 # To node found
                 if to_node:
@@ -803,32 +1070,9 @@ def copy_or_shift_logic(
 
                 # To node not found
                 else:
-                    # Find parent node
-                    to_path_list = to_path.split(tree_sep)
-                    idx = 1
-                    to_path_parent = tree_sep.join(to_path_list[:-idx])
-                    if transfer_indicator:
-                        to_node = find_path(to_tree, to_path_parent)
-                    else:
-                        to_node = find_path(tree, to_path_parent)
-
-                    # Create intermediate parent node, if applicable
-                    while (not to_node) & (idx + 1 < len(to_path_list)):
-                        idx += 1
-                        to_path_parent = tree_sep.join(to_path_list[:-idx])
-                        if transfer_indicator:
-                            to_node = find_path(to_tree, to_path_parent)
-                        else:
-                            to_node = find_path(tree, to_path_parent)
-                    if not to_node:
-                        raise NotFoundError(
-                            f"Unable to find to_path {to_path}\n"
-                            f"Please specify valid path to shift node to"
-                        )
-                    for depth in range(len(to_path_list) - idx, len(to_path_list) - 1):
-                        intermediate_child_node = node_type(to_path_list[depth])
-                        intermediate_child_node.parent = to_node
-                        to_node = intermediate_child_node
+                    # Find parent node, create intermediate parent node if applicable
+                    to_path_parent = tree_sep.join(to_path.split(tree_sep)[:-1])
+                    to_node = add_path_to_tree(to_tree, to_path_parent, sep=tree_sep)
 
             # Reassign from_node to new parent
             if copy:
@@ -853,3 +1097,139 @@ def copy_or_shift_logic(
                 if delete_children:
                     del from_node.children
                 from_node.parent = to_node
+
+
+def replace_logic(
+    tree: Node,
+    from_paths: List[str],
+    to_paths: List[str],
+    sep: str = "/",
+    copy: bool = False,
+    skippable: bool = False,
+    delete_children: bool = False,
+    to_tree: Optional[Node] = None,
+    with_full_path: bool = False,
+) -> None:
+    """Shift or copy nodes from `from_paths` to *replace* `to_paths` *in-place*.
+
+    - Creates intermediate nodes if to path is not present
+    - Able to copy node, defaults to False (nodes are shifted; not copied).
+    - Able to skip nodes if from path is not found, defaults to False (from-nodes must be found; not skippable)
+    - Able to replace node only and delete children, defaults to False (nodes are shifted/copied together with children).
+    - Able to shift/copy nodes from one tree to another tree, defaults to None (shifting/copying happens within same tree)
+
+    For paths in `from_paths` and `to_paths`,
+      - Path name can be with or without leading tree path separator symbol.
+
+    For paths in `from_paths`,
+      - Path name can be partial path (trailing part of path) or node name.
+      - If ``with_full_path=True``, path name must be full path.
+      - Path name must be unique to one node.
+
+    For paths in `to_paths`,
+      - Path name must be full path.
+      - Path must exist, node-to-be-replaced must be present.
+
+    Args:
+        tree (Node): tree to modify
+        from_paths (List[str]): original paths to shift nodes from
+        to_paths (List[str]): new paths to shift nodes to
+        sep (str): path separator for input paths, applies to `from_path` and `to_path`
+        copy (bool): indicator to copy node, defaults to False
+        skippable (bool): indicator to skip if from path is not found, defaults to False
+        delete_children (bool): indicator to shift/copy node only without children, defaults to False
+        to_tree (Node): tree to copy to, defaults to None
+        with_full_path (bool): indicator to shift/copy node with full path in `from_paths`, results in faster search,
+            defaults to False
+    """
+    if not (isinstance(from_paths, list) and isinstance(to_paths, list)):
+        raise ValueError(
+            "Invalid type, `from_paths` and `to_paths` should be list type"
+        )
+    if len(from_paths) != len(to_paths):
+        raise ValueError(
+            f"Paths are different length, input `from_paths` have {len(from_paths)} entries, "
+            f"while output `to_paths` have {len(to_paths)} entries"
+        )
+
+    # Modify `sep` of from_paths and to_paths
+    if not to_tree:
+        to_tree = tree
+    tree_sep = to_tree.sep
+    from_paths = [path.rstrip(sep).replace(sep, tree.sep) for path in from_paths]
+    to_paths = [
+        path.rstrip(sep).replace(sep, tree_sep) if path else None for path in to_paths
+    ]
+
+    if with_full_path:
+        if not all(
+            [
+                path.lstrip(tree.sep).split(tree.sep)[0] == tree.root.node_name
+                for path in from_paths
+            ]
+        ):
+            raise ValueError(
+                "Invalid path in `from_paths` not starting with the root node. "
+                "Check your `from_paths` parameter, alternatively set `with_full_path=False` to shift "
+                "partial path instead of full path."
+            )
+    if not all(
+        [
+            path.lstrip(tree_sep).split(tree_sep)[0] == to_tree.root.node_name
+            for path in to_paths
+            if path
+        ]
+    ):
+        raise ValueError(
+            "Invalid path in `to_paths` not starting with the root node. Check your `to_paths` parameter."
+        )
+
+    # Perform shifting/copying to replace destination node
+    for from_path, to_path in zip(from_paths, to_paths):
+        if with_full_path:
+            from_node = find_full_path(tree, from_path)
+        else:
+            from_node = find_path(tree, from_path)
+
+        # From node not found
+        if not from_node:
+            if not skippable:
+                raise NotFoundError(
+                    f"Unable to find from_path {from_path}\n"
+                    f"Set `skippable` to True to skip shifting for nodes not found"
+                )
+            else:
+                logging.info(f"Unable to find from_path {from_path}")
+
+        # From node found
+        else:
+            to_node = find_full_path(to_tree, to_path)
+
+            # To node found
+            if to_node:
+                if from_node == to_node:
+                    raise TreeError(
+                        f"Attempting to replace the same node {from_node.node_name}\n"
+                        f"Check from path {from_path} and to path {to_path}"
+                    )
+
+            # To node not found
+            else:
+                raise NotFoundError(f"Unable to find to_path {to_path}")
+
+            # Replace to_node with from_node
+            if copy:
+                logging.debug(f"Copying {from_node.node_name}")
+                from_node = from_node.copy()
+            if delete_children:
+                del from_node.children
+            parent = to_node.parent
+            to_node_siblings = parent.children
+            to_node_idx = to_node_siblings.index(to_node)
+            for node in to_node_siblings[to_node_idx:]:
+                if node == to_node:
+                    to_node.parent = None
+                    from_node.parent = parent
+                else:
+                    node.parent = None
+                    node.parent = parent
