@@ -16,6 +16,9 @@ if TYPE_CHECKING:
     from bigtree.node.binarynode import BinaryNode
     from bigtree.node.dagnode import DAGNode
 
+    BaseNodeT = TypeVar("BaseNodeT", bound=BaseNode)
+    BinaryNodeT = TypeVar("BinaryNodeT", bound=BinaryNode)
+    DAGNodeT = TypeVar("DAGNodeT", bound=DAGNode)
     T = TypeVar("T", bound=Union[BaseNode, DAGNode])
 
 __all__ = [
@@ -24,26 +27,28 @@ __all__ = [
     "postorder_iter",
     "levelorder_iter",
     "levelordergroup_iter",
+    "zigzag_iter",
+    "zigzaggroup_iter",
     "dag_iterator",
 ]
 
 
 def inorder_iter(
-    tree: BinaryNode,
-    filter_condition: Optional[Callable[[BinaryNode], bool]] = None,
+    tree: BinaryNodeT,
+    filter_condition: Optional[Callable[[BinaryNodeT], bool]] = None,
     max_depth: int = 0,
-) -> Iterable[BinaryNode]:
+) -> Iterable[BinaryNodeT]:
     """Iterate through all children of a tree.
 
-    In Iteration Algorithm, LNR
+    In-Order Iteration Algorithm, LNR
         1. Recursively traverse the current node's left subtree.
         2. Visit the current node.
         3. Recursively traverse the current node's right subtree.
 
-    >>> from bigtree import BinaryNode, list_to_binarytree, inorder_iter, print_tree
+    >>> from bigtree import BinaryNode, list_to_binarytree, inorder_iter
     >>> num_list = [1, 2, 3, 4, 5, 6, 7, 8]
     >>> root = list_to_binarytree(num_list)
-    >>> print_tree(root)
+    >>> root.show()
     1
     ├── 2
     │   ├── 4
@@ -93,10 +98,10 @@ def preorder_iter(
 
     It is topologically sorted because a parent node is processed before its child nodes.
 
-    >>> from bigtree import Node, list_to_tree, preorder_iter, print_tree
+    >>> from bigtree import Node, list_to_tree, preorder_iter
     >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
     >>> root = list_to_tree(path_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -112,7 +117,7 @@ def preorder_iter(
     >>> [node.node_name for node in preorder_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
     ['a', 'd', 'e', 'g', 'f']
 
-    >>> [node.node_name for node in preorder_iter(root, stop_condition=lambda x: x.node_name=="e")]
+    >>> [node.node_name for node in preorder_iter(root, stop_condition=lambda x: x.node_name == "e")]
     ['a', 'b', 'd', 'c', 'f']
 
     >>> [node.node_name for node in preorder_iter(root, max_depth=3)]
@@ -141,11 +146,11 @@ def preorder_iter(
 
 
 def postorder_iter(
-    tree: BaseNode,
-    filter_condition: Optional[Callable[[BaseNode], bool]] = None,
-    stop_condition: Optional[Callable[[BaseNode], bool]] = None,
+    tree: BaseNodeT,
+    filter_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    stop_condition: Optional[Callable[[BaseNodeT], bool]] = None,
     max_depth: int = 0,
-) -> Iterable[BaseNode]:
+) -> Iterable[BaseNodeT]:
     """Iterate through all children of a tree.
 
     Post-Order Iteration Algorithm, LRN
@@ -153,10 +158,10 @@ def postorder_iter(
         2. Recursively traverse the current node's right subtree.
         3. Visit the current node.
 
-    >>> from bigtree import Node, list_to_tree, postorder_iter, print_tree
+    >>> from bigtree import Node, list_to_tree, postorder_iter
     >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
     >>> root = list_to_tree(path_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -172,7 +177,7 @@ def postorder_iter(
     >>> [node.node_name for node in postorder_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
     ['d', 'g', 'e', 'f', 'a']
 
-    >>> [node.node_name for node in postorder_iter(root, stop_condition=lambda x: x.node_name=="e")]
+    >>> [node.node_name for node in postorder_iter(root, stop_condition=lambda x: x.node_name == "e")]
     ['d', 'b', 'f', 'c', 'a']
 
     >>> [node.node_name for node in postorder_iter(root, max_depth=3)]
@@ -203,20 +208,20 @@ def postorder_iter(
 
 
 def levelorder_iter(
-    tree: BaseNode,
-    filter_condition: Optional[Callable[[BaseNode], bool]] = None,
-    stop_condition: Optional[Callable[[BaseNode], bool]] = None,
+    tree: BaseNodeT,
+    filter_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    stop_condition: Optional[Callable[[BaseNodeT], bool]] = None,
     max_depth: int = 0,
-) -> Iterable[BaseNode]:
+) -> Iterable[BaseNodeT]:
     """Iterate through all children of a tree.
 
-    Level Order Algorithm
+    Level-Order Iteration Algorithm
         1. Recursively traverse the nodes on same level.
 
-    >>> from bigtree import Node, list_to_tree, levelorder_iter, print_tree
+    >>> from bigtree import Node, list_to_tree, levelorder_iter
     >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
     >>> root = list_to_tree(path_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -232,7 +237,7 @@ def levelorder_iter(
     >>> [node.node_name for node in levelorder_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
     ['a', 'd', 'e', 'f', 'g']
 
-    >>> [node.node_name for node in levelorder_iter(root, stop_condition=lambda x: x.node_name=="e")]
+    >>> [node.node_name for node in levelorder_iter(root, stop_condition=lambda x: x.node_name == "e")]
     ['a', 'b', 'c', 'd', 'f']
 
     >>> [node.node_name for node in levelorder_iter(root, max_depth=3)]
@@ -250,7 +255,7 @@ def levelorder_iter(
         (Iterable[BaseNode])
     """
 
-    def _levelorder_iter(trees: List[BaseNode]) -> Iterable[BaseNode]:
+    def _levelorder_iter(trees: List[BaseNodeT]) -> Iterable[BaseNodeT]:
         next_level = []
         for _tree in trees:
             if _tree:
@@ -267,20 +272,20 @@ def levelorder_iter(
 
 
 def levelordergroup_iter(
-    tree: BaseNode,
-    filter_condition: Optional[Callable[[BaseNode], bool]] = None,
-    stop_condition: Optional[Callable[[BaseNode], bool]] = None,
+    tree: BaseNodeT,
+    filter_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    stop_condition: Optional[Callable[[BaseNodeT], bool]] = None,
     max_depth: int = 0,
-) -> Iterable[Iterable[BaseNode]]:
+) -> Iterable[Iterable[BaseNodeT]]:
     """Iterate through all children of a tree.
 
-    Level Order Group Algorithm
+    Level-Order Group Iteration Algorithm
         1. Recursively traverse the nodes on same level, returns nodes level by level in a nested list.
 
-    >>> from bigtree import Node, list_to_tree, levelordergroup_iter, print_tree
+    >>> from bigtree import Node, list_to_tree, levelordergroup_iter
     >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
     >>> root = list_to_tree(path_list)
-    >>> print_tree(root)
+    >>> root.show()
     a
     ├── b
     │   ├── d
@@ -296,7 +301,7 @@ def levelordergroup_iter(
     >>> [[node.node_name for node in group] for group in levelordergroup_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
     [['a'], [], ['d', 'e', 'f'], ['g']]
 
-    >>> [[node.node_name for node in group] for group in levelordergroup_iter(root, stop_condition=lambda x: x.node_name=="e")]
+    >>> [[node.node_name for node in group] for group in levelordergroup_iter(root, stop_condition=lambda x: x.node_name == "e")]
     [['a'], ['b', 'c'], ['d', 'f']]
 
     >>> [[node.node_name for node in group] for group in levelordergroup_iter(root, max_depth=3)]
@@ -314,24 +319,168 @@ def levelordergroup_iter(
         (Iterable[Iterable[BaseNode]])
     """
 
-    def _levelordergroup_iter(trees: List[BaseNode]) -> Iterable[Iterable[BaseNode]]:
+    def _levelordergroup_iter(trees: List[BaseNodeT]) -> Iterable[Iterable[BaseNodeT]]:
         current_tree = []
-        next_tree = []
+        next_level = []
         for _tree in trees:
             if (not max_depth or not _tree.depth > max_depth) and (
                 not stop_condition or not stop_condition(_tree)
             ):
                 if not filter_condition or filter_condition(_tree):
                     current_tree.append(_tree)
-                next_tree.extend([_child for _child in _tree.children if _child])
+                next_level.extend([_child for _child in _tree.children if _child])
         yield tuple(current_tree)
-        if len(next_tree) and (not max_depth or not next_tree[0].depth > max_depth):
-            yield from _levelordergroup_iter(next_tree)
+        if len(next_level) and (not max_depth or not next_level[0].depth > max_depth):
+            yield from _levelordergroup_iter(next_level)
 
     yield from _levelordergroup_iter([tree])
 
 
-def dag_iterator(dag: DAGNode) -> Iterable[Tuple[DAGNode, DAGNode]]:
+def zigzag_iter(
+    tree: BaseNodeT,
+    filter_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    stop_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    max_depth: int = 0,
+) -> Iterable[BaseNodeT]:
+    """Iterate through all children of a tree.
+
+    ZigZag Iteration Algorithm
+        1. Recursively traverse the nodes on same level, in a zigzag manner across different levels.
+
+    >>> from bigtree import Node, list_to_tree, zigzag_iter
+    >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
+    >>> root = list_to_tree(path_list)
+    >>> root.show()
+    a
+    ├── b
+    │   ├── d
+    │   └── e
+    │       ├── g
+    │       └── h
+    └── c
+        └── f
+
+    >>> [node.node_name for node in zigzag_iter(root)]
+    ['a', 'c', 'b', 'd', 'e', 'f', 'h', 'g']
+
+    >>> [node.node_name for node in zigzag_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
+    ['a', 'd', 'e', 'f', 'g']
+
+    >>> [node.node_name for node in zigzag_iter(root, stop_condition=lambda x: x.node_name == "e")]
+    ['a', 'c', 'b', 'd', 'f']
+
+    >>> [node.node_name for node in zigzag_iter(root, max_depth=3)]
+    ['a', 'c', 'b', 'd', 'e', 'f']
+
+    Args:
+        tree (BaseNode): input tree
+        filter_condition (Optional[Callable[[BaseNode], bool]]): function that takes in node as argument, optional
+            Return node if condition evaluates to `True`
+        stop_condition (Optional[Callable[[BaseNode], bool]]): function that takes in node as argument, optional
+            Stops iteration if condition evaluates to `True`
+        max_depth (int): maximum depth of iteration, based on `depth` attribute, defaults to None
+
+    Returns:
+        (Iterable[BaseNode])
+    """
+
+    def _zigzag_iter(
+        trees: List[BaseNodeT], reverse_indicator: bool = False
+    ) -> Iterable[BaseNodeT]:
+        next_level = []
+        for _tree in trees:
+            if _tree:
+                if (not max_depth or not _tree.depth > max_depth) and (
+                    not stop_condition or not stop_condition(_tree)
+                ):
+                    if not filter_condition or filter_condition(_tree):
+                        yield _tree
+                    next_level_nodes = list(_tree.children)
+                    if reverse_indicator:
+                        next_level_nodes = next_level_nodes[::-1]
+                    next_level.extend(next_level_nodes)
+        if len(next_level):
+            yield from _zigzag_iter(
+                next_level[::-1], reverse_indicator=not reverse_indicator
+            )
+
+    yield from _zigzag_iter([tree])
+
+
+def zigzaggroup_iter(
+    tree: BaseNodeT,
+    filter_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    stop_condition: Optional[Callable[[BaseNodeT], bool]] = None,
+    max_depth: int = 0,
+) -> Iterable[Iterable[BaseNodeT]]:
+    """Iterate through all children of a tree.
+
+    ZigZag Group Iteration Algorithm
+        1. Recursively traverse the nodes on same level, in a zigzag manner across different levels,
+            returns nodes level by level in a nested list.
+
+    >>> from bigtree import Node, list_to_tree, zigzaggroup_iter
+    >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
+    >>> root = list_to_tree(path_list)
+    >>> root.show()
+    a
+    ├── b
+    │   ├── d
+    │   └── e
+    │       ├── g
+    │       └── h
+    └── c
+        └── f
+
+    >>> [[node.node_name for node in group] for group in zigzaggroup_iter(root)]
+    [['a'], ['c', 'b'], ['d', 'e', 'f'], ['h', 'g']]
+
+    >>> [[node.node_name for node in group] for group in zigzaggroup_iter(root, filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"])]
+    [['a'], [], ['d', 'e', 'f'], ['g']]
+
+    >>> [[node.node_name for node in group] for group in zigzaggroup_iter(root, stop_condition=lambda x: x.node_name == "e")]
+    [['a'], ['c', 'b'], ['d', 'f']]
+
+    >>> [[node.node_name for node in group] for group in zigzaggroup_iter(root, max_depth=3)]
+    [['a'], ['c', 'b'], ['d', 'e', 'f']]
+
+    Args:
+        tree (BaseNode): input tree
+        filter_condition (Optional[Callable[[BaseNode], bool]]): function that takes in node as argument, optional
+            Return node if condition evaluates to `True`
+        stop_condition (Optional[Callable[[BaseNode], bool]]): function that takes in node as argument, optional
+            Stops iteration if condition evaluates to `True`
+        max_depth (int): maximum depth of iteration, based on `depth` attribute, defaults to None
+
+    Returns:
+        (Iterable[Iterable[BaseNode]])
+    """
+
+    def _zigzaggroup_iter(
+        trees: List[BaseNodeT], reverse_indicator: bool = False
+    ) -> Iterable[Iterable[BaseNodeT]]:
+        current_tree = []
+        next_level = []
+        for _tree in trees:
+            if (not max_depth or not _tree.depth > max_depth) and (
+                not stop_condition or not stop_condition(_tree)
+            ):
+                if not filter_condition or filter_condition(_tree):
+                    current_tree.append(_tree)
+                next_level_nodes = [_child for _child in _tree.children if _child]
+                if reverse_indicator:
+                    next_level_nodes = next_level_nodes[::-1]
+                next_level.extend(next_level_nodes)
+        yield tuple(current_tree)
+        if len(next_level) and (not max_depth or not next_level[0].depth > max_depth):
+            yield from _zigzaggroup_iter(
+                next_level[::-1], reverse_indicator=not reverse_indicator
+            )
+
+    yield from _zigzaggroup_iter([tree])
+
+
+def dag_iterator(dag: DAGNodeT) -> Iterable[Tuple[DAGNodeT, DAGNodeT]]:
     """Iterate through all nodes of a Directed Acyclic Graph (DAG).
     Note that node names must be unique.
     Note that DAG must at least have two nodes to be shown on graph.
@@ -357,7 +506,7 @@ def dag_iterator(dag: DAGNode) -> Iterable[Tuple[DAGNode, DAGNode]]:
     """
     visited_nodes = set()
 
-    def recursively_parse_dag(node: DAGNode) -> Iterable[Tuple[DAGNode, DAGNode]]:
+    def recursively_parse_dag(node: DAGNodeT) -> Iterable[Tuple[DAGNodeT, DAGNodeT]]:
         node_name = node.node_name
         visited_nodes.add(node_name)
 
