@@ -27,9 +27,13 @@ extensions = [
     "autodocsumm",
     "myst_parser",
     "sphinxemoji.sphinxemoji",
+    "sphinx.ext.mathjax",
 ]
 autodoc_default_options = {"autosummary": True}
 sphinxemoji_style = "twemoji"
+myst_enable_extensions = [
+    "attrs_block",
+]
 
 templates_path = ["_templates"]
 exclude_patterns = []
