@@ -8,6 +8,7 @@ from bigtree.tree.helper import clone_tree
 from bigtree.utils.exceptions import LoopError, TreeError
 from bigtree.utils.iterators import preorder_iter
 from tests.conftest import assert_print_statement
+from tests.test_constants import Constants
 
 
 class BaseNode2(BaseNode):
@@ -82,55 +83,23 @@ class TestBaseNode(unittest.TestCase):
         self.g.parent = self.e
         self.h.parent = self.e
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
         assert_print_statement(print, "BaseNode(age=90, name=a)\n", self.a)
 
-    def test_set_parent_error(self):
-        with pytest.raises(ValueError) as exc_info:
+    def test_set_parents_error(self):
+        with pytest.raises(AttributeError) as exc_info:
             self.b.parents = [self.a]
-        assert str(exc_info.value).startswith("Attempting to set `parents` attribute")
+        assert str(exc_info.value) == Constants.ERROR_NODE_SET_PARENTS_ATTR
 
-        with pytest.raises(ValueError) as exc_info:
+        with pytest.raises(AttributeError) as exc_info:
             self.b = BaseNode(parents=[self.a])
-        assert str(exc_info.value).startswith("Attempting to set `parents` attribute")
+        assert str(exc_info.value) == Constants.ERROR_NODE_SET_PARENTS_ATTR
 
-        with pytest.raises(ValueError) as exc_info:
+        with pytest.raises(AttributeError) as exc_info:
             self.b.parents
-        assert str(exc_info.value).startswith(
-            "Attempting to access `parents` attribute"
-        )
-
-    def test_set_parent_reassign(self):
-        self.a.children = [self.b, self.c]
-        self.d.children = [self.e]
-        self.b.parent = self.d
-        assert list(self.a.children) == [
-            self.c
-        ], f"Node a children, expected {[self.c]}, received {self.a.children}"
-        assert list(self.d.children) == [
-            self.e,
-            self.b,
-        ], f"Node d children, expected {[self.e, self.b]}, received {self.d.children}"
-        assert (
-            self.b.parent == self.d
-        ), f"Node b parent, expected {self.d}, received {self.b.parent}"
-
-    def test_set_children_reassign(self):
-        self.a.children = [self.c]
-        self.b.parent = self.a
-        self.d.children = [self.e, self.b]
-        assert list(self.a.children) == [
-            self.c
-        ], f"Node a children, expected {[self.c]}, received {self.a.children}"
-        assert list(self.d.children) == [
-            self.e,
-            self.b,
-        ], f"Node d children, expected {[self.e, self.b]}, received {self.d.children}"
-        assert (
-            self.b.parent == self.d
-        ), f"Node b parent, expected {self.d}, received {self.b.parent}"
+        assert str(exc_info.value) == Constants.ERROR_NODE_GET_PARENTS_ATTR
 
     def test_set_parent(self):
         self.b.parent = self.a
@@ -141,7 +110,7 @@ class TestBaseNode(unittest.TestCase):
         self.g.parent = self.e
         self.h.parent = self.e
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -157,7 +126,7 @@ class TestBaseNode(unittest.TestCase):
         self.a.sort(key=lambda x: x.name)
         self.b.sort(key=lambda x: x.name)
         self.e.sort(key=lambda x: x.name)
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -170,7 +139,7 @@ class TestBaseNode(unittest.TestCase):
         self.e >> self.g
         self.e >> self.h
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -183,7 +152,7 @@ class TestBaseNode(unittest.TestCase):
         self.g << self.e
         self.h << self.e
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -197,7 +166,7 @@ class TestBaseNode(unittest.TestCase):
         self.g = BaseNode(name="g", age=10, parent=self.e)
         self.h = BaseNode(name="h", age=6, parent=self.e)
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -217,19 +186,32 @@ class TestBaseNode(unittest.TestCase):
         dummy.parent = None
         assert not len(list(self.h.children))
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
+    def test_set_parent_reassign(self):
+        self.a.children = [self.b, self.c]
+        self.d.children = [self.e]
+        self.b.parent = self.d
+        assert list(self.a.children) == [
+            self.c
+        ], f"Node a children, expected {[self.c]}, received {self.a.children}"
+        assert list(self.d.children) == [
+            self.e,
+            self.b,
+        ], f"Node d children, expected {[self.e, self.b]}, received {self.d.children}"
+        assert (
+            self.b.parent == self.d
+        ), f"Node b parent, expected {self.d}, received {self.b.parent}"
+
     def test_set_parent_duplicate(self):
-        # Set parent again
         self.b.parent = self.a
         self.b.parent = self.a
         assert list(self.a.children) == [self.b]
         assert self.b.parent == self.a
 
     def test_set_parent_duplicate_constructor(self):
-        # Set parent again
         self.a = BaseNode(name="a", age=90)
         self.b = BaseNode(name="b", age=65, parent=self.a)
         self.b.parent = self.a
@@ -242,7 +224,32 @@ class TestBaseNode(unittest.TestCase):
         self.c.children = [self.f]
         self.e.children = [self.g, self.h]
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
+        assert_tree_structure_basenode_root_attr(self.a)
+        assert_tree_structure_basenode_self(self)
+
+    def test_set_children_append(self):
+        self.a.children = [self.b]
+        self.b.children = [self.d]
+        self.e.children = [self.g, self.h]
+        self.a.append(self.c)
+        self.b.append(self.e)
+        self.c.append(self.f)
+
+        assert_tree_structure_basenode_root(self.a)
+        assert_tree_structure_basenode_root_attr(self.a)
+        assert_tree_structure_basenode_self(self)
+
+    def test_set_children_extend(self):
+        self.a.children = [self.b]
+        self.b.children = [self.d]
+        self.c.children = [self.f]
+
+        self.a.extend([self.c])
+        self.b.extend([self.e])
+        self.e.extend([self.g, self.h])
+
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -255,7 +262,7 @@ class TestBaseNode(unittest.TestCase):
         self.a.sort(key=lambda x: x.name)
         self.b.sort(key=lambda x: x.name)
         self.e.sort(key=lambda x: x.name)
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -269,7 +276,7 @@ class TestBaseNode(unittest.TestCase):
         self.b = BaseNode(name="b", age=65, children=[self.d, self.e])
         self.a = BaseNode(name="a", age=90, children=[self.b, self.c])
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -284,7 +291,7 @@ class TestBaseNode(unittest.TestCase):
         assert dummy.parent == self.h
         self.h.children = []
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
@@ -299,23 +306,40 @@ class TestBaseNode(unittest.TestCase):
         assert dummy.parent == self.h
         dummy.parent.children = []
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
 
-    def test_set_children_none_parent(self):
-        with pytest.raises(TypeError):
-            self.h.children = None
+    def test_set_children_none_parent_error(self):
+        children = None
+        with pytest.raises(TypeError) as exc_info:
+            self.h.children = children
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="List or Tuple or Set", input_type=type(children)
+        )
+
+    def test_set_children_reassign(self):
+        self.a.children = [self.c]
+        self.b.parent = self.a
+        self.d.children = [self.e, self.b]
+        assert list(self.a.children) == [
+            self.c
+        ], f"Node a children, expected {[self.c]}, received {self.a.children}"
+        assert list(self.d.children) == [
+            self.e,
+            self.b,
+        ], f"Node d children, expected {[self.e, self.b]}, received {self.d.children}"
+        assert (
+            self.b.parent == self.d
+        ), f"Node b parent, expected {self.d}, received {self.b.parent}"
 
     def test_set_children_duplicate(self):
-        # Set child again
         self.a.children = [self.b]
         self.a.children = [self.b]
         assert list(self.a.children) == [self.b]
         assert self.b.parent == self.a
 
     def test_set_children_duplicate_constructor(self):
-        # Set child again
         self.a = BaseNode(children=[self.b])
         self.a.children = [self.b]
         assert list(self.a.children) == [self.b]
@@ -335,7 +359,7 @@ class TestBaseNode(unittest.TestCase):
             not a2.children[0] == self.b and not a2.children[1] == self.b
         ), "Copy does not copy child nodes"
         assert not a2.children == [self.b, self.c], "Copy does not copy child nodes"
-        assert_tree_structure_basenode_root_generic(a2)
+        assert_tree_structure_basenode_root(a2)
         assert_tree_structure_basenode_root_attr(a2)
 
     def test_shallow_copy_set_children(self):
@@ -352,45 +376,78 @@ class TestBaseNode(unittest.TestCase):
             a2.children[0] == self.b or a2.children[1] == self.b
         ), "Shallow copy does not copy child nodes"
         assert a2.children == (self.b, self.c), "Shallow copy does not copy child nodes"
-        assert_tree_structure_basenode_root_generic(a2)
+        assert_tree_structure_basenode_root(a2)
         assert_tree_structure_basenode_root_attr(a2)
 
-    def test_error_set_parent_type_error(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.parent = 1
+    def test_set_parent_type_error(self):
+        parent = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.parent = parent
+        assert str(exc_info.value) == Constants.ERROR_NODE_PARENT_TYPE_NONE.format(
+            type="BaseNode", input_type=type(parent)
+        )
 
-    def test_error_set_parent_loop_error(self):
-        # Error: set self as parent
-        with pytest.raises(LoopError):
+    def test_set_parent_loop_error(self):
+        with pytest.raises(LoopError) as exc_info:
             self.a.parent = self.a
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_PARENT
 
-        # Error: set descendant as parent
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.b.parent = self.a
             self.c.parent = self.b
             self.a.parent = self.c
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_ANCESTOR
 
-    def test_error_set_children_type_error(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.children = [self.b, 1]
+    def test_set_children_mutable_list(self):
+        children_list = [self.b, self.c, self.d]
+        self.a.children = children_list
+        children_list.pop()
+        actual_children = self.a.children
+        expected_children = (self.b, self.c, self.d)
+        assert (
+            actual_children == expected_children
+        ), f"Expected {expected_children}, Received {actual_children}"
 
-    def test_error_set_children_loop_error(self):
-        # Error: set self as child
-        with pytest.raises(LoopError):
+    def test_set_children_iterable(self):
+        self.a.children = (self.b, self.c)
+        self.b.children = [self.d, self.e]
+        self.c.children = {self.f}
+        self.e.children = (self.g, self.h)
+
+        assert_tree_structure_basenode_root(self.a)
+        assert_tree_structure_basenode_root_attr(self.a)
+        assert_tree_structure_basenode_self(self)
+
+    def test_set_children_type_error(self):
+        children = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [self.b, children]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="BaseNode", input_type=type(children)
+        )
+
+        children = None
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [self.b, children]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="BaseNode", input_type=type(children)
+        )
+
+    def test_set_children_loop_error(self):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.a]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_CHILD
 
-        # Error: set ancestor as child
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.c]
             self.c.children = [self.d, self.e, self.f]
             self.f.children = [self.a]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_DESCENDANT
 
-    def test_error_set_children_exception(self):
-        # Error: duplicate child
-        with pytest.raises(TreeError):
+    def test_set_duplicate_children_error(self):
+        with pytest.raises(TreeError) as exc_info:
             self.a.children = [self.b, self.b]
+        assert str(exc_info.value) == Constants.ERROR_NODE_DUPLICATE_CHILD
 
     def test_rollback_set_parent(self):
         a = clone_tree(self.a, BaseNode2)
@@ -408,7 +465,7 @@ class TestBaseNode(unittest.TestCase):
         a.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             f.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [b, c]
         expected_h_children = [e, f, g]
@@ -439,7 +496,7 @@ class TestBaseNode(unittest.TestCase):
         a.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             d.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [b, c]
         expected_h_children = [e, f, g]
@@ -470,7 +527,7 @@ class TestBaseNode(unittest.TestCase):
         d.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             d.parent = None
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value) == "Custom error assigning parent"
 
         expected_a_children = [b, c]
         expected_h_children = [e, f, g]
@@ -501,7 +558,7 @@ class TestBaseNode(unittest.TestCase):
         a.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             b.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [b, c, d]
         expected_h_children = [e, f, g]
@@ -532,7 +589,7 @@ class TestBaseNode(unittest.TestCase):
         g.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             a.children = [b, c, d, g, i, f]
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value).startswith("Custom error assigning children, ")
         assert not i.parent, f"Node i parent, expected {None}, received {i.parent}"
 
         expected_a_children = [b, c, d]
@@ -564,7 +621,7 @@ class TestBaseNode(unittest.TestCase):
         a.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             a.children = []
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value) == "Custom error assigning children"
         assert not i.parent, f"Node i parent, expected {None}, received {i.parent}"
 
         expected_a_children = [b, c, d]
@@ -596,7 +653,7 @@ class TestBaseNode(unittest.TestCase):
         b.set_attrs({"val": 1})
         with pytest.raises(TreeError) as exc_info:
             a.children = [b, c, d]
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value).startswith("Custom error assigning children, ")
         assert not i.parent, f"Node i parent, expected {None}, received {i.parent}"
 
         expected_a_children = [b, c, d]
@@ -612,7 +669,8 @@ class TestBaseNode(unittest.TestCase):
                     ), f"Node {child} parent, expected {parent}, received {child.parent}"
 
 
-def assert_tree_structure_basenode_root_generic(root):
+def assert_tree_structure_basenode_root(root):
+    """Test tree structure (i.e., ancestors, descendants, leaves, siblings, etc.)"""
     # Test ancestors
     expected = 0
     actual = len(list(root.ancestors))
@@ -722,9 +780,10 @@ def assert_tree_structure_basenode_root_attr(
     g=("g", 10),
     h=("h", 6),
 ):
+    """Test tree structure with age attributes"""
     # Test describe()
-    expected = [("age", 90), ("name", "a")]
-    actual = root.describe(exclude_prefix="_")
+    expected = {("age", 90), ("name", "a")}
+    actual = set(root.describe(exclude_prefix="_"))
     assert (
         actual == expected
     ), f"Node description should be {expected}, but it is {actual}"
@@ -738,9 +797,49 @@ def assert_tree_structure_basenode_root_attr(
         ), f"Node name and age should be {expected}, but it is {actual}"
 
 
+def assert_tree_structure_customnode_root_attr(
+    root,
+    a=("a", 90),
+    b=("b", 65),
+    c=("c", 60),
+    d=("d", 40),
+    e=("e", 35),
+    f=("f", 38),
+    g=("g", 10),
+    h=("h", 6),
+):
+    """Test tree structure with age attributes"""
+    # Test describe()
+    expected = {("custom_field", 90), ("custom_field_str", "a"), ("name", "a")}
+    actual = set(root.describe(exclude_prefix="_"))
+    assert (
+        actual == expected
+    ), f"Node description should be {expected}, but it is {actual}"
+
+    # Test age attribute
+    expected_attrs = [a, b, d, e, g, h, c, f]
+    for node, expected in zip(preorder_iter(root), expected_attrs):
+        actual = node.get_attr("custom_field_str"), node.get_attr("custom_field")
+        assert (
+            actual == expected
+        ), f"Node custom_field_str and custom_field should be {expected}, but it is {actual}"
+
+
 def assert_tree_structure_basenode_self(self):
+    """Test tree structure with self object"""
     nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
 
+    # Test iteration (__iter__)
+    expected = [self.b, self.c]
+    actual = [child for child in self.a]
+    assert (
+        actual == expected
+    ), f"Node {self.a} should have {expected} children when iterated, but it has {actual}"
+
+    # Test contains (__contains__)
+    assert self.b in self.a, f"Check if {self.a} contains {self.b}, expected True"
+    assert self.d not in self.a, f"Check if {self.a} contains {self.d}, expected False"
+
     # Test ancestors
     expected_ans = [0, 1, 1, 2, 2, 2, 3, 3]
     for node, expected in zip(nodes, expected_ans):
