@@ -8,6 +8,7 @@ from bigtree.node.node import Node
 from bigtree.tree.helper import clone_tree
 from bigtree.utils.exceptions import LoopError, TreeError
 from tests.conftest import assert_print_statement
+from tests.test_constants import Constants
 
 
 class BinaryNode2(BinaryNode):
@@ -89,80 +90,18 @@ class TestBinaryNode(unittest.TestCase):
             self.b.path_name == "\\1\\2"
         ), f"Expected \\1\\2, received {self.b.path_name}"
 
-    def test_set_parent_error(self):
-        with pytest.raises(ValueError) as exc_info:
+    def test_set_parents_error(self):
+        with pytest.raises(AttributeError) as exc_info:
             self.b.parents = [self.a]
-        assert str(exc_info.value).startswith("Attempting to set `parents` attribute")
+        assert str(exc_info.value) == Constants.ERROR_NODE_SET_PARENTS_ATTR
 
-        with pytest.raises(ValueError) as exc_info:
+        with pytest.raises(AttributeError) as exc_info:
             self.b = BinaryNode(1, parents=[self.a])
-        assert str(exc_info.value).startswith("Attempting to set `parents` attribute")
+        assert str(exc_info.value) == Constants.ERROR_NODE_SET_PARENTS_ATTR
 
-        with pytest.raises(ValueError) as exc_info:
+        with pytest.raises(AttributeError) as exc_info:
             self.b.parents
-        assert str(exc_info.value).startswith(
-            "Attempting to access `parents` attribute"
-        )
-
-    def test_set_parent_3_parent_error(self):
-        self.b.parent = self.a
-        self.c.parent = self.a
-        with pytest.raises(TreeError) as exc_info:
-            self.d.parent = self.a
-        assert str(exc_info.value).endswith("already has 2 children")
-
-    def test_set_parent_position(self):
-        self.a.children = [self.b, self.c]
-        self.b.parent = self.d
-        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
-        assert (
-            self.a.right == self.c
-        ), f"Node a right, expected {self.c}, received {self.a.right}"
-        assert (
-            self.d.left == self.b
-        ), f"Node d left, expected {self.b}, received {self.d.left}"
-        assert not self.d.right, f"Node d right, expected None, received {self.d.right}"
-
-    def test_set_parent_position_right(self):
-        self.a.children = [self.b, self.c]
-        self.e.parent = self.d
-        self.b.parent = self.d
-        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
-        assert (
-            self.a.right == self.c
-        ), f"Node a right, expected {self.c}, received {self.a.right}"
-        assert (
-            self.d.left == self.e
-        ), f"Node d left, expected {self.e}, received {self.d.left}"
-        assert (
-            self.d.right == self.b
-        ), f"Node d right, expected {self.b}, received {self.d.right}"
-
-    def test_set_children_position(self):
-        self.a.children = [self.b, self.c]
-        self.d.children = [self.b, None]
-        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
-        assert (
-            self.a.right == self.c
-        ), f"Node a right, expected {self.c}, received {self.a.right}"
-        assert (
-            self.d.left == self.b
-        ), f"Node d left, expected {self.b}, received {self.d.left}"
-        assert not self.d.right, f"Node d right, expected None, received {self.d.right}"
-
-    def test_set_children_position_right(self):
-        self.a.children = [self.b, self.c]
-        self.d.children = [self.e, self.b]
-        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
-        assert (
-            self.a.right == self.c
-        ), f"Node a right, expected {self.c}, received {self.a.right}"
-        assert (
-            self.d.left == self.e
-        ), f"Node d left, expected {self.e}, received {self.d.left}"
-        assert (
-            self.d.right == self.b
-        ), f"Node d right, expected {self.b}, received {self.d.right}"
+        assert str(exc_info.value) == Constants.ERROR_NODE_GET_PARENTS_ATTR
 
     def test_set_parent(self):
         self.b.parent = self.a
@@ -175,6 +114,13 @@ class TestBinaryNode(unittest.TestCase):
 
         assert_binarytree_structure_self(self)
 
+    def test_set_parent_3_parent_error(self):
+        self.b.parent = self.a
+        self.c.parent = self.a
+        with pytest.raises(TreeError) as exc_info:
+            self.d.parent = self.a
+        assert str(exc_info.value).endswith("already has 2 children")
+
     def test_set_parent_children_insert(self):
         self.a.children = [None, None]
         self.b.parent = self.a
@@ -257,15 +203,40 @@ class TestBinaryNode(unittest.TestCase):
         dummy.parent = None
         assert_binarytree_structure_self(self)
 
+    def test_set_parent_reassign(self):
+        self.a.children = [self.b, self.c]
+        self.b.parent = self.d
+        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
+        assert (
+            self.a.right == self.c
+        ), f"Node a right, expected {self.c}, received {self.a.right}"
+        assert (
+            self.d.left == self.b
+        ), f"Node d left, expected {self.b}, received {self.d.left}"
+        assert not self.d.right, f"Node d right, expected None, received {self.d.right}"
+
+    def test_set_parent_reassign_right(self):
+        self.a.children = [self.b, self.c]
+        self.e.parent = self.d
+        self.b.parent = self.d
+        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
+        assert (
+            self.a.right == self.c
+        ), f"Node a right, expected {self.c}, received {self.a.right}"
+        assert (
+            self.d.left == self.e
+        ), f"Node d left, expected {self.e}, received {self.d.left}"
+        assert (
+            self.d.right == self.b
+        ), f"Node d right, expected {self.b}, received {self.d.right}"
+
     def test_set_parent_duplicate(self):
-        # Set parent again
         self.b.parent = self.a
         self.b.parent = self.a
         assert list(self.a.children) == [self.b, None]
         assert self.b.parent == self.a
 
     def test_set_parent_duplicate_constructor(self):
-        # Set parent again
         self.a = BinaryNode(1)
         self.b = BinaryNode(2, parent=self.a)
         self.b.parent = self.a
@@ -310,22 +281,27 @@ class TestBinaryNode(unittest.TestCase):
     def test_set_left_and_right_and_children_constructor_error(self):
         with pytest.raises(ValueError) as exc_info:
             self.a = BinaryNode(1, left=self.b, right=self.c, children=[self.c])
-        assert str(exc_info.value).startswith("Children input must have length 2")
+        assert str(exc_info.value) == Constants.ERROR_BINARYNODE_CHILDREN_LENGTH
 
+        left = self.b
+        right = self.c
+        children = [self.d, self.c]
         with pytest.raises(ValueError) as exc_info:
-            self.a = BinaryNode(1, left=self.b, right=self.c, children=[self.d, self.c])
-        assert str(exc_info.value).startswith(
-            "Attempting to set both left and children with mismatched values"
+            self.a = BinaryNode(1, left=left, right=right, children=children)
+        assert str(exc_info.value) == Constants.ERROR_BINARYNODE_LEFT_CHILDREN.format(
+            left=left, children=children
         )
 
+        left = self.b
+        right = self.c
+        children = [self.b, self.d]
         with pytest.raises(ValueError) as exc_info:
-            self.a = BinaryNode(1, left=self.b, right=self.c, children=[self.b, self.d])
-        assert str(exc_info.value).startswith(
-            "Attempting to set both right and children with mismatched values"
+            self.a = BinaryNode(1, left=left, right=right, children=children)
+        assert str(exc_info.value) == Constants.ERROR_BINARYNODE_RIGHT_CHILDREN.format(
+            right=right, children=children
         )
 
     def test_set_left_and_right_duplicate(self):
-        # Set parent again
         self.a = BinaryNode(1, left=self.b, right=self.c)
         self.a.left = self.b
         self.a.right = self.c
@@ -333,11 +309,6 @@ class TestBinaryNode(unittest.TestCase):
         assert self.b.parent == self.a
         assert self.c.parent == self.a
 
-    def test_set_children_3_children_error(self):
-        with pytest.raises(ValueError) as exc_info:
-            self.a.children = [self.b, self.c, self.d]
-        assert str(exc_info.value) == "Children input must have length 2"
-
     def test_set_children(self):
         self.a.children = [self.b, self.c]
         self.b.children = [self.d, self.e]
@@ -346,6 +317,11 @@ class TestBinaryNode(unittest.TestCase):
 
         assert_binarytree_structure_self(self)
 
+    def test_set_children_3_children_error(self):
+        with pytest.raises(ValueError) as exc_info:
+            self.a.children = [self.b, self.c, self.d]
+        assert str(exc_info.value) == Constants.ERROR_BINARYNODE_CHILDREN_LENGTH
+
     def test_set_children_sort(self):
         self.a.children = [self.c, self.b]
         self.b.children = [self.e, self.d]
@@ -400,19 +376,47 @@ class TestBinaryNode(unittest.TestCase):
 
         assert_binarytree_structure_self(self)
 
-    def test_set_children_none_parent(self):
-        with pytest.raises(TypeError):
-            self.h.children = None
+    def test_set_children_none_parent_error(self):
+        children = None
+        with pytest.raises(TypeError) as exc_info:
+            self.h.children = children
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="List or Tuple or Set", input_type=type(children)
+        )
+
+    def test_set_children_reassign(self):
+        self.a.children = [self.b, self.c]
+        self.d.children = [self.b, None]
+        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
+        assert (
+            self.a.right == self.c
+        ), f"Node a right, expected {self.c}, received {self.a.right}"
+        assert (
+            self.d.left == self.b
+        ), f"Node d left, expected {self.b}, received {self.d.left}"
+        assert not self.d.right, f"Node d right, expected None, received {self.d.right}"
+
+    def test_set_children_reassign_right(self):
+        self.a.children = [self.b, self.c]
+        self.d.children = [self.e, self.b]
+        assert not self.a.left, f"Node a left, expected None, received {self.a.left}"
+        assert (
+            self.a.right == self.c
+        ), f"Node a right, expected {self.c}, received {self.a.right}"
+        assert (
+            self.d.left == self.e
+        ), f"Node d left, expected {self.e}, received {self.d.left}"
+        assert (
+            self.d.right == self.b
+        ), f"Node d right, expected {self.b}, received {self.d.right}"
 
     def test_set_children_duplicate(self):
-        # Set child again
         self.a.children = [self.b, None]
         self.a.children = [self.b, None]
         assert list(self.a.children) == [self.b, None]
         assert self.b.parent == self.a
 
     def test_set_children_duplicate_constructor(self):
-        # Set child again
         self.a = BinaryNode(1, children=[self.b, None])
         self.a.children = [self.b, None]
         assert list(self.a.children) == [self.b, None]
@@ -434,67 +438,88 @@ class TestBinaryNode(unittest.TestCase):
         assert not a2.children == [self.b, self.c], "Copy does not copy child nodes"
         assert_binarytree_structure_root(a2)
 
-    def test_error_set_parent_type_error(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.parent = 1
+    def test_set_parent_type_error(self):
+        parent = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.parent = parent
+        assert str(exc_info.value) == Constants.ERROR_NODE_PARENT_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(parent)
+        )
 
-        a = BaseNode()
-        with pytest.raises(TypeError):
-            self.a.parent = a
+        parent = BaseNode()
+        with pytest.raises(TypeError) as exc_info:
+            self.a.parent = parent
+        assert str(exc_info.value) == Constants.ERROR_NODE_PARENT_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(parent)
+        )
 
-        a = Node("a")
-        with pytest.raises(TypeError):
-            self.a.parent = a
+        parent = Node("a")
+        with pytest.raises(TypeError) as exc_info:
+            self.a.parent = parent
+        assert str(exc_info.value) == Constants.ERROR_NODE_PARENT_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(parent)
+        )
 
-    def test_error_set_parent_loop_error(self):
-        # Error: set self as parent
-        with pytest.raises(LoopError):
+    def test_set_parent_loop_error(self):
+        with pytest.raises(LoopError) as exc_info:
             self.a.parent = self.a
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_PARENT
 
-        # Error: set descendant as parent
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.b.parent = self.a
             self.c.parent = self.b
             self.a.parent = self.c
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_ANCESTOR
 
-    def test_error_set_children_type_error(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.children = self.b
+    def test_set_children_type_error(self):
+        children = self.b
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = children
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="List or Tuple or Set", input_type=type(children)
+        )
 
-        with pytest.raises(TypeError):
-            self.a.children = [self.b, 1]
+        children = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [self.b, children]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(children)
+        )
 
-        a = BaseNode()
-        with pytest.raises(TypeError):
-            self.a.children = [a, None]
+        children = BaseNode()
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [children, None]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(children)
+        )
 
-        a = Node("a")
-        with pytest.raises(TypeError):
-            self.a.children = [a, None]
+        children = Node("a")
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [children, None]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE_NONE.format(
+            type="BinaryNode", input_type=type(children)
+        )
 
-    def test_error_set_children_length_error(self):
-        # Error: wrong type
+    def test_set_children_length_error(self):
         with pytest.raises(ValueError) as exc_info:
             self.a.children = [self.b]
-        assert str(exc_info.value).startswith("Children input must have length 2")
+        assert str(exc_info.value) == Constants.ERROR_BINARYNODE_CHILDREN_LENGTH
 
-    def test_error_set_children_loop_error(self):
-        # Error: set self as child
-        with pytest.raises(LoopError):
+    def test_set_children_loop_error(self):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.a]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_CHILD
 
-        # Error: set ancestor as child
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.c]
             self.c.children = [self.d, self.e]
             self.e.children = [self.a, self.f]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_DESCENDANT
 
-    def test_error_set_children_exception(self):
-        # Error: duplicate child
-        with pytest.raises(TreeError):
+    def test_set_duplicate_children_error(self):
+        with pytest.raises(TreeError) as exc_info:
             self.a.children = [self.b, self.b]
+        assert str(exc_info.value) == Constants.ERROR_NODE_DUPLICATE_CHILD
 
     def test_rollback_set_parent(self):
         a = clone_tree(self.a, BinaryNode2)
@@ -512,7 +537,7 @@ class TestBinaryNode(unittest.TestCase):
         a.val = 100
         with pytest.raises(TreeError) as exc_info:
             b.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [None, c]
         expected_b_children = [d, e]
@@ -546,7 +571,7 @@ class TestBinaryNode(unittest.TestCase):
         a.val = 100
         with pytest.raises(TreeError) as exc_info:
             g.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [None, c]
         expected_b_children = [d, e]
@@ -581,7 +606,7 @@ class TestBinaryNode(unittest.TestCase):
         g.val = 100
         with pytest.raises(TreeError) as exc_info:
             g.parent = None
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value) == "Custom error assigning parent"
 
         expected_a_children = [None, c]
         expected_b_children = [d, e]
@@ -615,7 +640,7 @@ class TestBinaryNode(unittest.TestCase):
         a.val = 100
         with pytest.raises(TreeError) as exc_info:
             c.parent = a
-        assert str(exc_info.value).startswith("Custom error assigning parent")
+        assert str(exc_info.value).startswith("Custom error assigning parent, ")
 
         expected_a_children = [None, c]
         expected_b_children = [d, e]
@@ -646,7 +671,7 @@ class TestBinaryNode(unittest.TestCase):
         f.val = 100
         with pytest.raises(TreeError) as exc_info:
             a.children = [d, f]
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value).startswith("Custom error assigning children, ")
 
         expected_a_children = [b, c]
         expected_b_children = [d, e]
@@ -676,7 +701,7 @@ class TestBinaryNode(unittest.TestCase):
         a.val = 100
         with pytest.raises(TreeError) as exc_info:
             a.children = []
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value) == "Custom error assigning children"
 
         expected_a_children = [b, c]
         expected_b_children = [d, e]
@@ -704,7 +729,7 @@ class TestBinaryNode(unittest.TestCase):
         f.val = 100
         with pytest.raises(TreeError) as exc_info:
             a.children = [e, f]
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value).startswith("Custom error assigning children, ")
 
         expected_a_children = [b, None]
         expected_b_children = [None, e]
@@ -733,7 +758,7 @@ class TestBinaryNode(unittest.TestCase):
         b.val = 100
         with pytest.raises(TreeError) as exc_info:
             a.children = [b, c]
-        assert str(exc_info.value).startswith("Custom error assigning children")
+        assert str(exc_info.value).startswith("Custom error assigning children, ")
 
         expected_a_children = [b, c]
         expected_b_children = [d, e]
@@ -749,14 +774,38 @@ class TestBinaryNode(unittest.TestCase):
                     ), f"Node {child} parent, expected {parent}, received {child.parent}"
 
 
+def assert_binarytree_structure_root(root):
+    """Test tree structure"""
+    a = root
+    b, c = a.children
+    d, e = b.children
+    f, g = c.children
+    _, h = d.children
+
+    class Sample:
+        pass
+
+    self = Sample()
+    self.a = a
+    self.b = b
+    self.c = c
+    self.d = d
+    self.e = e
+    self.f = f
+    self.g = g
+    self.h = h
+    assert_binarytree_structure_self(self)
+
+
 def assert_binarytree_structure_self(self):
+    """Test tree structure with self object"""
     nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
 
     # Test parent
     expected_ans = [None, self.a, self.a, self.b, self.b, self.c, self.c, self.d]
     for node, expected in zip(nodes, expected_ans):
         actual = node.parent
-        assert expected == actual, f"Expected parent {expected}, received {actual}"
+        assert actual == expected, f"Expected parent {expected}, received {actual}"
 
     # Test children
     expected_ans = [
@@ -771,19 +820,19 @@ def assert_binarytree_structure_self(self):
     ]
     for node, expected in zip(nodes, expected_ans):
         actual = list(node.children)
-        assert expected == actual, f"Expected children {expected}, received {actual}"
+        assert actual == expected, f"Expected children {expected}, received {actual}"
 
     # Test left
     expected_ans = [self.b, self.d, self.f, None, None, None, None, None]
     for node, expected in zip(nodes, expected_ans):
         actual = node.left
-        assert expected == actual, f"Expected left {expected}, received {actual}"
+        assert actual == expected, f"Expected left {expected}, received {actual}"
 
     # Test right
     expected_ans = [self.c, self.e, self.g, self.h, None, None, None, None]
     for node, expected in zip(nodes, expected_ans):
         actual = node.right
-        assert expected == actual, f"Expected right {expected}, received {actual}"
+        assert actual == expected, f"Expected right {expected}, received {actual}"
 
     # Test ancestors
     expected_ans = [0, 1, 1, 2, 2, 2, 2, 3]
@@ -910,75 +959,11 @@ def assert_binarytree_structure_self(self):
     expected_ans = [1, 2, 3, 4, 5, 6, 7, 8]
     for node, expected in zip(nodes, expected_ans):
         actual = node.get_attr("val")
-        assert expected == actual, f"Expected right {expected}, received {actual}"
-
-
-def assert_binarytree_structure_root(root):
-    a = root
-    b, c = a.children
-    d, e = b.children
-    f, g = c.children
-    _, h = d.children
-
-    class Sample:
-        pass
-
-    self = Sample()
-    self.a = a
-    self.b = b
-    self.c = c
-    self.d = d
-    self.e = e
-    self.f = f
-    self.g = g
-    self.h = h
-    assert_binarytree_structure_self(self)
-
-
-def assert_binarytree_structure_self2(self):
-    nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
-
-    # Test parent
-    expected_parent = [None, self.a, self.a, self.b, self.b, self.c, self.c, self.d]
-    for node, expected in zip(nodes, expected_parent):
-        actual = node.parent
-        assert expected == actual, f"Expected parent {expected}, received {actual}"
-
-    # Test children
-    expected_children = [
-        [self.b, self.c],
-        [self.d, self.e],
-        [self.f, self.g],
-        [self.h, None],
-        [None, None],
-        [None, None],
-        [None, None],
-        [None, None],
-    ]
-    for node, expected in zip(nodes, expected_children):
-        actual = list(node.children)
-        assert expected == actual, f"Expected children {expected}, received {actual}"
-
-    # Test left
-    expected_left = [self.b, self.d, self.f, self.h, None, None, None, None]
-    for node, expected in zip(nodes, expected_left):
-        actual = node.left
-        assert expected == actual, f"Expected left {expected}, received {actual}"
-
-    # Test right
-    expected_right = [self.c, self.e, self.g, None, None, None, None, None]
-    for node, expected in zip(nodes, expected_right):
-        actual = node.right
-        assert expected == actual, f"Expected right {expected}, received {actual}"
-
-    # Test get_attr()
-    expected_val = [1, 2, 3, 4, 5, 6, 7, 8]
-    for node, expected in zip(nodes, expected_val):
-        actual = node.get_attr("val")
-        assert expected == actual, f"Expected attribute {expected}, received {actual}"
+        assert actual == expected, f"Expected right {expected}, received {actual}"
 
 
 def assert_binarytree_structure_root2(root):
+    """Test tree structure"""
     a = root
     b, c = a.children
     d, e = b.children
@@ -998,3 +983,47 @@ def assert_binarytree_structure_root2(root):
     self.g = g
     self.h = h
     assert_binarytree_structure_self2(self)
+
+
+def assert_binarytree_structure_self2(self):
+    """Test tree structure with self object"""
+    nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
+
+    # Test parent
+    expected_parent = [None, self.a, self.a, self.b, self.b, self.c, self.c, self.d]
+    for node, expected in zip(nodes, expected_parent):
+        actual = node.parent
+        assert actual == expected, f"Expected parent {expected}, received {actual}"
+
+    # Test children
+    expected_children = [
+        [self.b, self.c],
+        [self.d, self.e],
+        [self.f, self.g],
+        [self.h, None],
+        [None, None],
+        [None, None],
+        [None, None],
+        [None, None],
+    ]
+    for node, expected in zip(nodes, expected_children):
+        actual = list(node.children)
+        assert actual == expected, f"Expected children {expected}, received {actual}"
+
+    # Test left
+    expected_left = [self.b, self.d, self.f, self.h, None, None, None, None]
+    for node, expected in zip(nodes, expected_left):
+        actual = node.left
+        assert actual == expected, f"Expected left {expected}, received {actual}"
+
+    # Test right
+    expected_right = [self.c, self.e, self.g, None, None, None, None, None]
+    for node, expected in zip(nodes, expected_right):
+        actual = node.right
+        assert actual == expected, f"Expected right {expected}, received {actual}"
+
+    # Test get_attr()
+    expected_val = [1, 2, 3, 4, 5, 6, 7, 8]
+    for node, expected in zip(nodes, expected_val):
+        actual = node.get_attr("val")
+        assert actual == expected, f"Expected attribute {expected}, received {actual}"
