@@ -6,11 +6,13 @@ import pytest
 from bigtree.node.node import Node
 from bigtree.utils.exceptions import LoopError, TreeError
 from bigtree.utils.iterators import preorder_iter
+from tests.conftest import assert_print_statement
 from tests.node.test_basenode import (
+    assert_tree_structure_basenode_root,
     assert_tree_structure_basenode_root_attr,
-    assert_tree_structure_basenode_root_generic,
     assert_tree_structure_basenode_self,
 )
+from tests.test_constants import Constants
 
 
 class TestNode(unittest.TestCase):
@@ -45,9 +47,10 @@ class TestNode(unittest.TestCase):
         self.g = None
         self.h = None
 
-    def test_empty_node_name(self):
-        with pytest.raises(TreeError):
+    def test_empty_node_name_error(self):
+        with pytest.raises(TreeError) as exc_info:
             Node()
+        assert str(exc_info.value) == Constants.ERROR_NODE_NAME
 
     def test_set_parent(self):
         self.b.parent = self.a
@@ -58,10 +61,10 @@ class TestNode(unittest.TestCase):
         self.g.parent = self.e
         self.h.parent = self.e
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
-        assert_tree_structure_node_root_generic(self.a)
+        assert_tree_structure_node_root(self.a)
         assert_tree_structure_node_self(self)
 
     def test_set_parent_constructor(self):
@@ -74,10 +77,10 @@ class TestNode(unittest.TestCase):
         self.g = Node(name="g", age=10, parent=self.e)
         self.h = Node(name="h", age=6, parent=self.e)
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
-        assert_tree_structure_node_root_generic(self.a)
+        assert_tree_structure_node_root(self.a)
         assert_tree_structure_node_self(self)
 
     def test_set_parent_duplicate(self):
@@ -95,6 +98,16 @@ class TestNode(unittest.TestCase):
         assert list(self.a.children) == [self.b]
         assert self.b.parent == self.a
 
+    def test_set_parent_sep_different_error(self):
+        b = Node("b", sep="\\")
+        self.b.parent = self.a
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
+            b.parent = self.a
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_PARENT_PATH.format(
+            path=path
+        )
+
     def test_set_parent_sep_root(self):
         self.b.parent = self.a
         self.c.parent = self.a
@@ -105,7 +118,7 @@ class TestNode(unittest.TestCase):
         self.h.parent = self.e
         self.a.sep = "\\"
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
         assert_tree_structure_node_self_sep(self)
@@ -120,7 +133,7 @@ class TestNode(unittest.TestCase):
         self.h.parent = self.e
         self.h.sep = "\\"
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
         assert_tree_structure_node_self_sep(self)
@@ -131,10 +144,10 @@ class TestNode(unittest.TestCase):
         self.c.children = [self.f]
         self.e.children = [self.g, self.h]
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
-        assert_tree_structure_node_root_generic(self.a)
+        assert_tree_structure_node_root(self.a)
         assert_tree_structure_node_self(self)
 
     def test_set_children_constructor(self):
@@ -147,94 +160,127 @@ class TestNode(unittest.TestCase):
         self.b = Node(name="b", age=65, children=[self.d, self.e])
         self.a = Node(name="a", age=90, children=[self.b, self.c])
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
-        assert_tree_structure_node_root_generic(self.a)
+        assert_tree_structure_node_root(self.a)
         assert_tree_structure_node_self(self)
 
     def test_set_children_duplicate(self):
-        # Set child again
         self.a.children = [self.b]
         self.a.children = [self.b]
 
     def test_set_children_duplicate_constructor(self):
-        # Set child again
         self.a = Node("a", children=[self.b])
         self.a.children = [self.b]
 
-    def test_error_set_parent_same_path(self):
+    def test_set_children_sep_different_error(self):
+        b = Node("b", sep="\\")
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
+            self.a.children = [self.b, b]
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_CHILDREN_PATH.format(
+            path=path
+        )
+
+    def test_set_parent_same_path_error(self):
         self.a = Node("a")
         self.b = Node("b", parent=self.a)
         self.c = Node("b")
-        with pytest.raises(TreeError):
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
             self.c.parent = self.a
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_PARENT_PATH.format(
+            path=path
+        )
 
-    def test_error_set_parent_constructor_same_path(self):
+    def test_set_parent_constructor_same_path_error(self):
         self.a = Node("a")
         self.b = Node("b", parent=self.a)
-        with pytest.raises(TreeError):
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
             self.c = Node("b", parent=self.a)
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_PARENT_PATH.format(
+            path=path
+        )
 
-    def test_error_set_children_same_path(self):
+    def test_set_children_same_path_error(self):
         self.a = Node("a")
         self.b = Node("b")
         self.c = Node("b")
-        with pytest.raises(TreeError):
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
             self.a.children = [self.b, self.c]
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_CHILDREN_PATH.format(
+            path=path
+        )
 
-    def test_error_set_children_constructor_same_path(self):
+    def test_set_children_constructor_same_path_error(self):
         self.c = Node("b")
         self.b = Node("b")
-        with pytest.raises(TreeError):
+        path = "/a/b"
+        with pytest.raises(TreeError) as exc_info:
             self.a = Node("a", children=[self.b, self.c])
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_CHILDREN_PATH.format(
+            path=path
+        )
 
-    def test_error_set_children_multiple_same_path(self):
+    def test_set_children_multiple_same_path_error(self):
         self.a = Node("a")
         self.b = Node("b")
         self.c = Node("b")
         self.d = Node("c")
         self.e = Node("c")
+        path = "/a/b and /a/c"
         with pytest.raises(TreeError) as exc_info:
             self.a.children = [self.b, self.c, self.d, self.e]
-        assert (
-            str(exc_info.value)
-            == "Error: Duplicate node with same path\nAttempting to add nodes same path /a/b and /a/c"
+        assert str(exc_info.value) == Constants.ERROR_NODE_SAME_CHILDREN_PATH.format(
+            path=path
         )
 
-    def test_error_set_parent(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.parent = 1
+    def test_set_parent_error(self):
+        parent = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.parent = parent
+        assert str(exc_info.value) == Constants.ERROR_NODE_PARENT_TYPE_NONE.format(
+            type="BaseNode", input_type=type(parent)
+        )
 
-        # Error: set self as parent
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.a.parent = self.a
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_PARENT
 
-        # Error: set descendant as parent
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.b.parent = self.a
             self.c.parent = self.b
             self.a.parent = self.c
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_ANCESTOR
 
-    def test_error_set_children(self):
-        # Error: wrong type
-        with pytest.raises(TypeError):
-            self.a.children = [self.b, 1]
+    def test_set_children_error(self):
+        children = 1
+        with pytest.raises(TypeError) as exc_info:
+            self.a.children = [self.b, children]
+        assert str(exc_info.value) == Constants.ERROR_NODE_CHILDREN_TYPE.format(
+            type="BaseNode", input_type=type(children)
+        )
 
-        # Error: set self as child
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.a]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_CHILD
 
-        # Error: set ancestor as child
-        with pytest.raises(LoopError):
+        with pytest.raises(LoopError) as exc_info:
             self.a.children = [self.b, self.c]
             self.c.children = [self.d, self.e, self.f]
             self.f.children = [self.a]
+        assert str(exc_info.value) == Constants.ERROR_NODE_LOOP_DESCENDANT
 
-        # Error: duplicate child
-        with pytest.raises(TreeError):
+        with pytest.raises(TreeError) as exc_info:
             self.a.children = [self.b, self.b]
+        assert str(exc_info.value) == Constants.ERROR_NODE_DUPLICATE_CHILD
+
+    def test_path_name_int(self):
+        b = Node(1, parent=self.a)
+        assert b.path_name == "/a/1"
 
     def test_go_to(self):
         self.a.children = [self.b, self.c]
@@ -242,10 +288,10 @@ class TestNode(unittest.TestCase):
         self.c.children = [self.f]
         self.e.children = [self.g, self.h]
 
-        assert_tree_structure_basenode_root_generic(self.a)
+        assert_tree_structure_basenode_root(self.a)
         assert_tree_structure_basenode_root_attr(self.a)
         assert_tree_structure_basenode_self(self)
-        assert_tree_structure_node_root_generic(self.a)
+        assert_tree_structure_node_root(self.a)
         assert_tree_structure_node_self(self)
 
         expected_paths = [
@@ -296,18 +342,24 @@ class TestNode(unittest.TestCase):
 
     def test_go_to_type_error(self):
         a = Node("a")
+        destination = 2
         with pytest.raises(TypeError) as exc_info:
-            a.go_to(2)
-        assert str(exc_info.value).startswith("Expect node to be BaseNode type")
+            a.go_to(destination)
+        assert str(exc_info.value) == Constants.ERROR_NODE_GOTO_TYPE.format(
+            type="BaseNode", input_type=type(destination)
+        )
 
     def test_go_to_different_tree_error(self):
-        a = Node("a")
+        source = Node("a")
+        destination = self.a
         with pytest.raises(TreeError) as exc_info:
-            a.go_to(self.a)
-        assert str(exc_info.value).startswith("Nodes are not from the same tree")
+            source.go_to(destination)
+        assert str(exc_info.value) == Constants.ERROR_NODE_GOTO_SAME_TREE.format(
+            a=source, b=destination
+        )
 
 
-def assert_tree_structure_node_root_generic(
+def assert_tree_structure_node_root(
     root,
     a="/a",
     b="/a/b",
@@ -333,6 +385,31 @@ def assert_tree_structure_node_root_generic(
 def assert_tree_structure_node_self(self):
     nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
 
+    # Test accessing with square bracket (__getitem__)
+    assert (
+        self.a["b"] == self.b
+    ), f"Accessor method not returning correct for node {self.a} for {self.b}"
+    assert (
+        self.a["c"] == self.c
+    ), f"Accessor method not returning correct for node {self.a} for {self.c}"
+    assert (
+        self.a["b"]["d"] == self.d
+    ), f"Accessor method not returning correct for node {self.a} for {self.d}"
+
+    # Test deletion of children with square bracket (__delitem__)
+    assert len(self.a.children) == 2, f"Before deletion: error in {self.a.children}"
+    del self.a["c"]
+    assert len(self.a.children) == 1, f"After deletion: error in {self.a.children}"
+    self.c.parent = self.a
+    assert (
+        len(self.a.children) == 2
+    ), f"Revert after deletion: error in {self.a.children}"
+
+    # Test deletion of non-existent children with square bracket (__delitem__)
+    assert len(self.a.children) == 2, f"Before deletion: error in {self.a.children}"
+    del self.a["d"]
+    assert len(self.a.children) == 2, f"After deletion: error in {self.a.children}"
+
     # Test path_name
     expected_ans = [
         "/a",
@@ -350,6 +427,20 @@ def assert_tree_structure_node_self(self):
             actual == expected
         ), f"Node should have path {expected}, but path is {actual}"
 
+    # Test show
+    expected_str = """a\n├── b\n│   ├── d\n│   └── e\n│       ├── g\n│       └── h\n└── c\n    └── f\n"""
+    assert_print_statement(
+        self.a.show,
+        expected_str,
+    )
+
+    expected_str = """a\n|-- b\n|   |-- d\n|   `-- e\n|       |-- g\n|       `-- h\n`-- c\n    `-- f\n"""
+    assert_print_statement(
+        self.a.show,
+        expected_str,
+        style="ansi",
+    )
+
 
 def assert_tree_structure_node_root_sep(root):
     # Test path_name
