@@ -5,6 +5,8 @@ from bigtree.utils.iterators import (
     levelordergroup_iter,
     postorder_iter,
     preorder_iter,
+    zigzag_iter,
+    zigzaggroup_iter,
 )
 
 
@@ -13,13 +15,13 @@ class TestPreOrderIter:
     def test_preorder_iter(tree_node):
         expected = ["a", "b", "d", "e", "g", "h", "c", "f"]
         actual = [node.node_name for node in preorder_iter(tree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter2(tree_node2):
         expected = ["a", "b", "d", "g", "e", "h", "i", "c", "f"]
         actual = [node.node_name for node in preorder_iter(tree_node2)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_filter_condition(tree_node):
@@ -31,7 +33,7 @@ class TestPreOrderIter:
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_filter_condition_skip_level(tree_node):
@@ -43,7 +45,7 @@ class TestPreOrderIter:
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_stop_condition(tree_node):
@@ -54,13 +56,13 @@ class TestPreOrderIter:
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_max_depth(tree_node):
         expected = ["a", "b", "d", "e", "c", "f"]
         actual = [node.node_name for node in preorder_iter(tree_node, max_depth=3)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
 class TestPostOrderIter:
@@ -68,13 +70,13 @@ class TestPostOrderIter:
     def test_postorder_iter(tree_node):
         expected = ["d", "g", "h", "e", "b", "f", "c", "a"]
         actual = [node.node_name for node in postorder_iter(tree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter2(tree_node2):
         expected = ["g", "d", "h", "i", "e", "b", "f", "c", "a"]
         actual = [node.node_name for node in postorder_iter(tree_node2)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter_filter_condition(tree_node):
@@ -86,7 +88,7 @@ class TestPostOrderIter:
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter_filter_condition_skip_level(tree_node):
@@ -98,7 +100,7 @@ class TestPostOrderIter:
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter_stop_condition(tree_node):
@@ -109,13 +111,13 @@ class TestPostOrderIter:
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter_max_depth(tree_node):
         expected = ["d", "e", "b", "f", "c", "a"]
         actual = [node.node_name for node in postorder_iter(tree_node, max_depth=3)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
 class TestLevelOrderIter:
@@ -123,13 +125,13 @@ class TestLevelOrderIter:
     def test_levelorder_iter(tree_node):
         expected = ["a", "b", "c", "d", "e", "f", "g", "h"]
         actual = [node.node_name for node in levelorder_iter(tree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter2(tree_node2):
         expected = ["a", "b", "c", "d", "e", "f", "g", "h", "i"]
         actual = [node.node_name for node in levelorder_iter(tree_node2)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter_filter_condition(tree_node):
@@ -141,7 +143,7 @@ class TestLevelOrderIter:
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter_filter_condition_skip_level(tree_node):
@@ -153,7 +155,7 @@ class TestLevelOrderIter:
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter_stop_condition(tree_node):
@@ -164,13 +166,13 @@ class TestLevelOrderIter:
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter_max_depth(tree_node):
         expected = ["a", "b", "c", "d", "e", "f"]
         actual = [node.node_name for node in levelorder_iter(tree_node, max_depth=3)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
 class TestLevelOrderGroupIter:
@@ -181,7 +183,7 @@ class TestLevelOrderGroupIter:
             [node.node_name for node in group]
             for group in levelordergroup_iter(tree_node)
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelordergroup_iter2(tree_node2):
@@ -190,7 +192,7 @@ class TestLevelOrderGroupIter:
             [node.node_name for node in group]
             for group in levelordergroup_iter(tree_node2)
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelordergroup_iter_filter_condition(tree_node):
@@ -202,7 +204,7 @@ class TestLevelOrderGroupIter:
                 filter_condition=lambda x: x.name in ["a", "d", "e", "f", "g"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelordergroup_iter_filter_condition_skip_level(tree_node):
@@ -214,7 +216,7 @@ class TestLevelOrderGroupIter:
                 filter_condition=lambda x: x.name in ["g", "h"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelordergroup_iter_stop_condition(tree_node):
@@ -225,7 +227,7 @@ class TestLevelOrderGroupIter:
                 tree_node, stop_condition=lambda x: x.name == "e"
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelordergroup_iter_max_depth(tree_node):
@@ -234,7 +236,124 @@ class TestLevelOrderGroupIter:
             [node.node_name for node in group]
             for group in levelordergroup_iter(tree_node, max_depth=3)
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+
+class TestZigZagIter:
+    @staticmethod
+    def test_zigzag_iter(tree_node):
+        expected = ["a", "c", "b", "d", "e", "f", "h", "g"]
+        actual = [node.node_name for node in zigzag_iter(tree_node)]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter2(tree_node2):
+        expected = ["a", "c", "b", "d", "e", "f", "i", "h", "g"]
+        actual = [node.node_name for node in zigzag_iter(tree_node2)]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_filter_condition(tree_node):
+        expected = ["a", "d", "e", "f", "g"]
+        actual = [
+            node.node_name
+            for node in zigzag_iter(
+                tree_node,
+                filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_filter_condition_skip_level(tree_node):
+        expected = ["h", "g"]
+        actual = [
+            node.node_name
+            for node in zigzag_iter(
+                tree_node,
+                filter_condition=lambda x: x.node_name in ["g", "h"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_stop_condition(tree_node):
+        expected = ["a", "c", "b", "d", "f"]
+        actual = [
+            node.node_name
+            for node in zigzag_iter(
+                tree_node, stop_condition=lambda x: x.node_name == "e"
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_max_depth(tree_node):
+        expected = ["a", "c", "b", "d", "e", "f"]
+        actual = [node.node_name for node in zigzag_iter(tree_node, max_depth=3)]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+
+class TestZigZagGroupIter:
+    @staticmethod
+    def test_zigzaggroup_iter(tree_node):
+        expected = [["a"], ["c", "b"], ["d", "e", "f"], ["h", "g"]]
+        actual = [
+            [node.node_name for node in group] for group in zigzaggroup_iter(tree_node)
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter2(tree_node2):
+        expected = [["a"], ["c", "b"], ["d", "e", "f"], ["i", "h", "g"]]
+        actual = [
+            [node.node_name for node in group] for group in zigzaggroup_iter(tree_node2)
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_filter_condition(tree_node):
+        expected = [["a"], [], ["d", "e", "f"], ["g"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(
+                tree_node,
+                filter_condition=lambda x: x.name in ["a", "d", "e", "f", "g"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_filter_condition_skip_level(tree_node):
+        expected = [[], [], [], ["h", "g"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(
+                tree_node,
+                filter_condition=lambda x: x.name in ["g", "h"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_stop_condition(tree_node):
+        expected = [["a"], ["c", "b"], ["d", "f"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(
+                tree_node, stop_condition=lambda x: x.name == "e"
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_max_depth(tree_node):
+        expected = [["a"], ["c", "b"], ["d", "e", "f"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(tree_node, max_depth=3)
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
 class TestDAGIterator:
@@ -257,7 +376,7 @@ class TestDAGIterator:
         ]
         len_expected = 9
         len_actual = len(actual)
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
         assert (
             len_expected == len_actual
         ), f"Expected\n{len_expected}\nReceived\n{len_actual}"
@@ -281,7 +400,7 @@ class TestDAGIterator:
         ]
         len_expected = 9
         len_actual = len(actual)
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
         assert (
             len_expected == len_actual
         ), f"Expected\n{len_expected}\nReceived\n{len_actual}"
@@ -292,7 +411,7 @@ class TestBinaryTreeIterator:
     def test_preorder_iter(binarytree_node):
         expected = ["1", "2", "4", "8", "5", "3", "6", "7"]
         actual = [node.node_name for node in preorder_iter(binarytree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_filter_condition(binarytree_node):
@@ -304,7 +423,7 @@ class TestBinaryTreeIterator:
                 filter_condition=lambda x: x.node_name in ["1", "3", "6", "4"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_stop_condition(binarytree_node):
@@ -315,7 +434,7 @@ class TestBinaryTreeIterator:
                 binarytree_node, stop_condition=lambda x: x.node_name == "3"
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter_max_depth(binarytree_node):
@@ -323,34 +442,78 @@ class TestBinaryTreeIterator:
         actual = [
             node.node_name for node in preorder_iter(binarytree_node, max_depth=3)
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter(binarytree_node):
         expected = ["8", "4", "5", "2", "6", "7", "3", "1"]
         actual = [node.node_name for node in postorder_iter(binarytree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter(binarytree_node):
         expected = ["1", "2", "3", "4", "5", "6", "7", "8"]
         actual = [node.node_name for node in levelorder_iter(binarytree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
-    def test_levelordergroup_iter2(binarytree_node):
+    def test_levelorder_iter_filter_condition(binarytree_node):
+        expected = ["1", "3", "4", "6", "7"]
+        actual = [
+            node.node_name
+            for node in levelorder_iter(
+                binarytree_node,
+                filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_levelorder_iter_max_depth(binarytree_node):
+        expected = ["1", "2", "3", "4", "5", "6", "7"]
+        actual = [
+            node.node_name for node in levelorder_iter(binarytree_node, max_depth=3)
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_levelordergroup_iter(binarytree_node):
         expected = [["1"], ["2", "3"], ["4", "5", "6", "7"], ["8"]]
         actual = [
             [node.node_name for node in group]
             for group in levelordergroup_iter(binarytree_node)
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_levelordergroup_iter_filter_condition(binarytree_node):
+        expected = [["1"], ["3"], ["4", "6", "7"], []]
+        actual = [
+            [node.node_name for node in group]
+            for group in levelordergroup_iter(
+                binarytree_node,
+                filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_levelordergroup_iter_max_depth(binarytree_node):
+        expected = [["1"], ["2", "3"], ["4", "5", "6", "7"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in levelordergroup_iter(
+                binarytree_node,
+                max_depth=3,
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_inorder_iter(binarytree_node):
         expected = ["4", "8", "2", "5", "1", "6", "3", "7"]
         actual = [node.node_name for node in inorder_iter(binarytree_node)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_inorder_iter_filter_condition(binarytree_node):
@@ -362,10 +525,67 @@ class TestBinaryTreeIterator:
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
         ]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_inorder_iter_max_depth(binarytree_node):
         expected = ["4", "2", "5", "1", "6", "3", "7"]
         actual = [node.node_name for node in inorder_iter(binarytree_node, max_depth=3)]
-        assert expected == actual, f"Expected\n{expected}\nReceived\n{actual}"
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter(binarytree_node):
+        expected = ["1", "3", "2", "4", "5", "6", "7", "8"]
+        actual = [node.node_name for node in zigzag_iter(binarytree_node)]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_filter_condition(binarytree_node):
+        expected = ["1", "3", "4", "6", "7"]
+        actual = [
+            node.node_name
+            for node in zigzag_iter(
+                binarytree_node,
+                filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzag_iter_max_depth(binarytree_node):
+        expected = ["1", "3", "2", "4", "5", "6", "7"]
+        actual = [node.node_name for node in zigzag_iter(binarytree_node, max_depth=3)]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter(binarytree_node):
+        expected = [["1"], ["3", "2"], ["4", "5", "6", "7"], ["8"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(binarytree_node)
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_filter_condition(binarytree_node):
+        expected = [["1"], ["3"], ["4", "6", "7"], []]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(
+                binarytree_node,
+                filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+    @staticmethod
+    def test_zigzaggroup_iter_max_depth(binarytree_node):
+        expected = [["1"], ["3", "2"], ["4", "5", "6", "7"]]
+        actual = [
+            [node.node_name for node in group]
+            for group in zigzaggroup_iter(
+                binarytree_node,
+                max_depth=3,
+            )
+        ]
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
